# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.command_modules.acs._client_factory import (
    cf_agent_pools,
    cf_managed_clusters,
    cf_maintenance_configurations,
    cf_snapshots,
    cf_trustedaccess_role,
    cf_trustedaccess_role_binding,
    cf_machines
)
from azure.cli.command_modules.acs._format import (
    aks_agentpool_list_table_format,
    aks_agentpool_show_table_format,
    aks_list_nodepool_snapshot_table_format,
    aks_list_table_format,
    aks_run_command_result_format,
    aks_show_nodepool_snapshot_table_format,
    aks_show_table_format,
    aks_upgrades_table_format,
    aks_versions_table_format,
    aks_mesh_revisions_table_format,
    aks_mesh_upgrades_table_format,
    aks_machine_list_table_format,
    aks_machine_show_table_format,
)
from azure.cli.core.commands import CliCommandType
from azure.cli.core.profiles import ResourceType


# pylint: disable=too-many-statements
# pylint: disable=line-too-long
def load_command_table(self, _):

    managed_clusters_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.containerservice.operations.'
                        '_managed_clusters_operations#ManagedClustersOperations.{}',
        operation_group='managed_clusters',
        resource_type=ResourceType.MGMT_CONTAINERSERVICE,
        client_factory=cf_managed_clusters
    )

    agent_pools_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.containerservice.operations.'
                        '_agent_pools_operations#AgentPoolsOperations.{}',
        operation_group='agent_pools',
        resource_type=ResourceType.MGMT_CONTAINERSERVICE,
        client_factory=cf_managed_clusters
    )

    machines_sdk = CliCommandType(
        operations_tmpl='azext_aks_preview.vendored_sdks.azure_mgmt_preview_aks.'
                        'operations._machine_operations#MachinesOperations.{}',
        client_factory=cf_managed_clusters
    )

    maintenance_configuration_sdk = CliCommandType(
        operations_tmpl='aazure.mgmt.containerservice.operations.'
                        '_maintenance_configurations_operations#MaintenanceConfigurationsOperations.{}',
        operation_group='maintenance_configurations',
        resource_type=ResourceType.MGMT_CONTAINERSERVICE,
        client_factory=cf_maintenance_configurations
    )

    snapshot_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.containerservice.operations.'
                        '_snapshots_operations#SnapshotsOperations.{}',
        operation_group='snapshots',
        resource_type=ResourceType.MGMT_CONTAINERSERVICE,
        client_factory=cf_snapshots
    )

    trustedaccess_role_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.containerservice.operations.'
                        '_trusted_access_roles_operations#TrustedAccessRolesOperations.{}',
        operation_group='trustedaccess_role',
        resource_type=ResourceType.MGMT_CONTAINERSERVICE,
        client_factory=cf_trustedaccess_role
    )

    trustedaccess_role_binding_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.containerservice.operations.'
                        '_trusted_access_role_bindings_operations#TrustedAccessRoleBindingsOperations.{}',
        operation_group='trustedaccess_role_binding',
        resource_type=ResourceType.MGMT_CONTAINERSERVICE,
        client_factory=cf_trustedaccess_role_binding
    )

    # AKS commands
    with self.command_group('aks', managed_clusters_sdk,
                            client_factory=cf_managed_clusters) as g:
        g.custom_command('browse', 'aks_browse')
        g.custom_command('create', 'aks_create', supports_no_wait=True)
        g.custom_command('update', 'aks_update', supports_no_wait=True)
        g.command('get-upgrades', 'get_upgrade_profile',
                  table_transformer=aks_upgrades_table_format)
        g.custom_command('upgrade', 'aks_upgrade', supports_no_wait=True)
        g.custom_command('scale', 'aks_scale', supports_no_wait=True)
        g.command('delete', 'begin_delete',
                  supports_no_wait=True, confirmation=True)
        g.custom_show_command('show', 'aks_show',
                              table_transformer=aks_show_table_format)
        g.custom_command('list', 'aks_list',
                         table_transformer=aks_list_table_format)
        g.custom_command('enable-addons', 'aks_enable_addons',
                         supports_no_wait=True)
        g.custom_command('disable-addons', 'aks_disable_addons',
                         supports_no_wait=True)
        g.custom_command('get-credentials', 'aks_get_credentials')
        g.custom_command('update-credentials',
                         'aks_update_credentials', supports_no_wait=True)
        g.custom_command('check-acr', 'aks_check_acr')
        g.custom_command('install-cli', 'k8s_install_cli', client_factory=None)
        g.custom_command('rotate-certs', 'aks_rotate_certs', supports_no_wait=True,
                         confirmation='Kubernetes will be unavailable during certificate rotation process.\n' +
                         'Are you sure you want to perform this operation?')
        g.custom_command('stop', 'aks_stop',
                         supports_no_wait=True)
        g.command('start', 'begin_start',
                  supports_no_wait=True)
        g.wait_command('wait')
        g.custom_command('use-dev-spaces', 'aks_use_dev_spaces',
                         deprecate_info=g.deprecate())
        g.custom_command('remove-dev-spaces',
                         'aks_remove_dev_spaces', deprecate_info=g.deprecate())
        g.custom_command('operation-abort',
                         'aks_operation_abort', supports_no_wait=True)
        g.custom_command('get-versions', 'aks_get_versions',
                         table_transformer=aks_versions_table_format)

    with self.command_group('aks machine', machines_sdk, client_factory=cf_machines) as g:
        g.custom_command('list', 'aks_machine_list',
                         table_transformer=aks_machine_list_table_format)
        g.custom_show_command('show', 'aks_machine_show',
                              table_transformer=aks_machine_show_table_format)

    # AKS maintenance configuration commands
    with self.command_group('aks maintenanceconfiguration', maintenance_configuration_sdk, client_factory=cf_maintenance_configurations) as g:
        g.custom_command('list', 'aks_maintenanceconfiguration_list')
        g.custom_show_command('show', 'aks_maintenanceconfiguration_show')
        g.custom_command('add', 'aks_maintenanceconfiguration_add')
        g.custom_command('update', 'aks_maintenanceconfiguration_update')
        g.custom_command('delete', 'aks_maintenanceconfiguration_delete')

    # AKS agent pool commands
    with self.command_group('aks nodepool',
                            agent_pools_sdk,
                            client_factory=cf_agent_pools) as g:
        g.custom_command('add', 'aks_agentpool_add', supports_no_wait=True)
        g.custom_command('update', 'aks_agentpool_update',
                         supports_no_wait=True)
        g.custom_command('get-upgrades', 'aks_agentpool_get_upgrade_profile')
        g.custom_command('upgrade', 'aks_agentpool_upgrade',
                         supports_no_wait=True)
        g.custom_command('scale', 'aks_agentpool_scale', supports_no_wait=True)
        g.custom_command('delete', 'aks_agentpool_delete',
                         supports_no_wait=True)
        g.custom_show_command('show', 'aks_agentpool_show',
                              table_transformer=aks_agentpool_show_table_format)
        g.custom_command('list', 'aks_agentpool_list',
                         table_transformer=aks_agentpool_list_table_format)
        g.custom_command('stop', 'aks_agentpool_stop', supports_no_wait=True)
        g.custom_command('start', 'aks_agentpool_start', supports_no_wait=True)
        g.wait_command('wait')
        g.custom_command(
            'operation-abort', 'aks_agentpool_operation_abort', supports_no_wait=True)
        g.custom_command(
            'delete-machines', 'aks_agentpool_delete_machines', supports_no_wait=True)

    # AKS nodepool manual-scale command
    with self.command_group('aks nodepool manual-scale', agent_pools_sdk, client_factory=cf_agent_pools) as g:
        g.custom_command("add", "aks_agentpool_manual_scale_add", supports_no_wait=True)
        g.custom_command("update", "aks_agentpool_manual_scale_update", supports_no_wait=True)
        g.custom_command("delete", "aks_agentpool_manual_scale_delete", supports_no_wait=True, confirmation=True)

    with self.command_group('aks command', managed_clusters_sdk, client_factory=cf_managed_clusters) as g:
        g.custom_command('invoke', 'aks_runcommand', supports_no_wait=True,
                         table_transformer=aks_run_command_result_format)
        g.custom_command('result', 'aks_command_result',
                         supports_no_wait=False, table_transformer=aks_run_command_result_format)

    # AKS nodepool snapshot commands
    with self.command_group('aks snapshot',
                            snapshot_sdk,
                            client_factory=cf_snapshots,
                            deprecate_info=self.deprecate(
                                redirect='aks nodepool snapshot', hide=True),
                            ) as g:
        g.custom_command('list', 'aks_nodepool_snapshot_list',
                         deprecate_info=g.deprecate(
                             redirect='aks nodepool snapshot list'),
                         table_transformer=aks_list_nodepool_snapshot_table_format)
        g.custom_show_command('show', 'aks_nodepool_snapshot_show',
                              deprecate_info=g.deprecate(
                                  redirect='aks nodepool snapshot show'),
                              table_transformer=aks_show_nodepool_snapshot_table_format)
        g.custom_command('create', 'aks_nodepool_snapshot_create',
                         deprecate_info=g.deprecate(redirect='aks nodepool snapshot create'), supports_no_wait=True)
        g.custom_command('delete', 'aks_nodepool_snapshot_delete',
                         deprecate_info=g.deprecate(redirect='aks nodepool snapshot delete'), supports_no_wait=True)
        g.wait_command('wait', deprecate_info=g.deprecate(
            redirect='aks nodepool snapshot wait'))

    with self.command_group('aks nodepool snapshot',
                            snapshot_sdk,
                            client_factory=cf_snapshots,
                            ) as g:
        g.custom_command('list', 'aks_nodepool_snapshot_list',
                         table_transformer=aks_list_nodepool_snapshot_table_format)
        g.custom_show_command('show', 'aks_nodepool_snapshot_show',
                              table_transformer=aks_show_nodepool_snapshot_table_format)
        g.custom_command(
            'create', 'aks_nodepool_snapshot_create', supports_no_wait=True)
        g.custom_command(
            'delete', 'aks_nodepool_snapshot_delete', supports_no_wait=True)
        g.custom_command('update', 'aks_nodepool_snapshot_update')
        g.wait_command('wait')

    with self.command_group('aks oidc-issuer', managed_clusters_sdk, client_factory=cf_managed_clusters) as g:
        g.custom_command('rotate-signing-keys', 'aks_rotate_service_account_signing_keys', supports_no_wait=True,
                         confirmation='Be careful that rotate oidc issuer signing keys twice within short period' +
                         ' will invalidate service accounts token immediately. Please refer to doc for details.\n' +
                         'Are you sure you want to perform this operation?')

    # AKS trusted access role commands
    with self.command_group('aks trustedaccess role', trustedaccess_role_sdk, client_factory=cf_trustedaccess_role) as g:
        g.custom_command('list', 'aks_trustedaccess_role_list')

    # AKS trusted access rolebinding commands
    with self.command_group('aks trustedaccess rolebinding', trustedaccess_role_binding_sdk, client_factory=cf_trustedaccess_role_binding) as g:
        g.custom_command('list', 'aks_trustedaccess_role_binding_list')
        g.custom_show_command('show', 'aks_trustedaccess_role_binding_get')
        g.custom_command('create', 'aks_trustedaccess_role_binding_create')
        g.custom_command('update', 'aks_trustedaccess_role_binding_update')
        g.custom_command(
            'delete', 'aks_trustedaccess_role_binding_delete', confirmation=True)

    # AKS mesh commands
    with self.command_group('aks mesh', managed_clusters_sdk, client_factory=cf_managed_clusters) as g:
        g.custom_command(
            'enable',
            'aks_mesh_enable',
            supports_no_wait=True)
        g.custom_command(
            "disable",
            "aks_mesh_disable",
            supports_no_wait=True,
            confirmation="Existing Azure Service Mesh Profile values will be reset.\n" +
            "Are you sure you want to perform this operation?")
        g.custom_command(
            'enable-ingress-gateway',
            'aks_mesh_enable_ingress_gateway',
            supports_no_wait=True)
        g.custom_command(
            'disable-ingress-gateway',
            'aks_mesh_disable_ingress_gateway',
            supports_no_wait=True,
            confirmation=True)
        g.custom_command(
            'get-revisions',
            'aks_mesh_get_revisions',
            table_transformer=aks_mesh_revisions_table_format)
        g.custom_command(
            'get-upgrades',
            'aks_mesh_get_upgrades',
            table_transformer=aks_mesh_upgrades_table_format)

    # AKS mesh upgrade commands
    with self.command_group('aks mesh upgrade', managed_clusters_sdk, client_factory=cf_managed_clusters) as g:
        g.custom_command(
            'start',
            'aks_mesh_upgrade_start',
            supports_no_wait=True)
        g.custom_command(
            'complete',
            'aks_mesh_upgrade_complete',
            supports_no_wait=True)
        g.custom_command(
            'rollback',
            'aks_mesh_upgrade_rollback',
            supports_no_wait=True)

    # AKS approuting commands
    with self.command_group('aks approuting', managed_clusters_sdk, client_factory=cf_managed_clusters) as g:
        g.custom_command('enable', 'aks_approuting_enable')
        g.custom_command('disable', 'aks_approuting_disable',
                         confirmation=True)
        g.custom_command('update', 'aks_approuting_update')

    # AKS approuting dns-zone commands
    with self.command_group('aks approuting zone', managed_clusters_sdk, client_factory=cf_managed_clusters) as g:
        g.custom_command('add', 'aks_approuting_zone_add')
        g.custom_command(
            'delete', 'aks_approuting_zone_delete', confirmation=True)
        g.custom_command('update', 'aks_approuting_zone_update')
        g.custom_command('list', 'aks_approuting_zone_list')

    with self.command_group('aks safeguards'):
        from .custom import AKSSafeguardsShowCustom as Show
        from .custom import AKSSafeguardsCreateCustom as Create
        from .custom import AKSSafeguardsUpdateCustom as Update
        from .custom import AKSSafeguardsDeleteCustom as Delete
        from .custom import AKSSafeguardsListCustom as List
        from .custom import AKSSafeguardsWaitCustom as Wait

        self.command_table["aks safeguards show"] = Show(loader=self)
        self.command_table["aks safeguards create"] = Create(loader=self)
        self.command_table["aks safeguards update"] = Update(loader=self)
        self.command_table["aks safeguards delete"] = Delete(loader=self)
        self.command_table["aks safeguards list"] = List(loader=self)
        self.command_table["aks safeguards wait"] = Wait(loader=self)



# coding=utf-8
# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


from knack.help_files import helps  # pylint: disable=unused-import

helps['login'] = """
type: command
short-summary: Log in to Azure.
long-summary: >-
    By default, this command logs in with a user account.
    Azure CLI uses Web Account Manager (WAM) on Windows, and browser-based login on Linux and macOS by default.
    If WAM or a web browser is not available, Azure CLI will fall back to device code login.


    [WARNING] Authentication with username and password in the command line is strongly discouraged.
    Use one of the recommended authentication methods based on your requirements.
    For more details, see https://go.microsoft.com/fwlink/?linkid=2276314


    [WARNING] `--password` no longer accepts a service principal certificate.
    Use `--certificate` to pass a service principal certificate.


    To log in with a service principal, specify --service-principal.


    To log in with a managed identity, specify --identity.


    For more details, see https://learn.microsoft.com/cli/azure/authenticate-azure-cli
examples:
    - name: Log in interactively.
      text: az login
    - name: Log in with username and password. This doesn't work with Microsoft accounts or accounts that have two-factor authentication enabled. Use -p=secret if the first character of the password is '-'.
      text: az login --username johndoe@contoso.com --password VerySecret
    - name: Log in with a service principal using client secret. Use --password=secret if the first character of the password is '-'.
      text: az login --service-principal --username APP_ID --password CLIENT_SECRET --tenant TENANT_ID
    - name: Log in with a service principal using certificate.
      text: az login --service-principal --username APP_ID --certificate /path/to/cert.pem --tenant TENANT_ID
    - name: Log in with a system-assigned managed identity.
      text: az login --identity
    - name: Log in with a user-assigned managed identity's client ID.
      text: az login --identity --client-id 00000000-0000-0000-0000-000000000000
    - name: Log in with a user-assigned managed identity's resource ID.
      text: az login --identity --resource-id /subscriptions/00000000-0000-0000-0000-000000000000/resourceGroups/MyResourceGroup/providers/Microsoft.ManagedIdentity/userAssignedIdentities/MyIdentity
"""

helps['account'] = """
type: group
short-summary: Manage Azure subscription information.
"""

helps['account clear'] = """
type: command
short-summary: Clear all subscriptions from the CLI's local cache.
long-summary: To clear the current subscription, use 'az logout'.
"""

helps['account list'] = """
type: command
short-summary: >-
    Get a list of subscriptions for the logged in account. By default, only 'Enabled' subscriptions from the current
    cloud is shown.
"""

helps['account show'] = """
type: command
short-summary: Get the details of a subscription.
long-summary: If no subscription is specified, shows the current subscription.
"""

helps['account set'] = """
type: command
short-summary: Set a subscription to be the current active subscription.
"""

helps['account show'] = """
type: command
short-summary: Get the details of a subscription.
long-summary: If the subscription isn't specified, shows the details of the default subscription.
"""

helps['account get-access-token'] = """
type: command
short-summary: Get a token for utilities to access Azure.
long-summary: >
    The token will be valid for at least 5 minutes with the maximum at 60 minutes.
    If the subscription argument isn't specified, the current account is used.


    In the output, `expires_on` represents a POSIX timestamp and `expiresOn` represents a local datetime.
    It is recommended for downstream applications to use `expires_on` because it is in UTC.
examples:
    - name: Get an access token for the current account
      text: >
        az account get-access-token
    - name: Get an access token for a specific subscription
      text: >
        az account get-access-token --subscription 00000000-0000-0000-0000-000000000000
    - name: Get an access token for a specific tenant
      text: >
        az account get-access-token --tenant 00000000-0000-0000-0000-000000000000
    - name: Get an access token to use with MS Graph API
      text: >
        az account get-access-token --resource-type ms-graph
"""

helps['self-test'] = """
type: command
short-summary: Runs a self-test of the CLI.
"""



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "monitor activity-log list-categories",
)
class ListCategories(AAZCommand):
    """List the list of available event categories supported in the Activity Logs Service.

    The current list includes the following: Administrative, Security, ServiceHealth, Alert, Recommendation, Policy.
    """

    _aaz_info = {
        "version": "2015-04-01",
        "resources": [
            ["mgmt-plane", "/providers/microsoft.insights/eventcategories", "2015-04-01"],
        ]
    }

    def _handler(self, command_args):
        super()._handler(command_args)
        self._execute_operations()
        return self._output()

    def _execute_operations(self):
        self.pre_operations()
        self.EventCategoriesList(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance.value, client_flatten=True)
        return result

    class EventCategoriesList(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/providers/Microsoft.Insights/eventcategories",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2015-04-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()

            _schema_on_200 = cls._schema_on_200
            _schema_on_200.value = AAZListType(
                flags={"required": True},
            )

            value = cls._schema_on_200.value
            value.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element
            _element.localized_value = AAZStrType(
                serialized_name="localizedValue",
            )
            _element.value = AAZStrType(
                flags={"required": True},
            )

            return cls._schema_on_200


class _ListCategoriesHelper:
    """Helper class for ListCategories"""


__all__ = ["ListCategories"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "compute-fleet list",
    is_preview=True,
)
class List(AAZCommand):
    """List Fleet resources by subscription ID and resource group

    List Fleet resources by subscription ID and resource group

    :example: Fleets_ListBySubscription
        az azure-fleet list
    """

    _aaz_info = {
        "version": "2024-11-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/providers/microsoft.azurefleet/fleets", "2024-11-01"],
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.azurefleet/fleets", "2024-11-01"],
        ]
    }

    AZ_SUPPORT_PAGINATION = True

    def _handler(self, command_args):
        super()._handler(command_args)
        return self.build_paging(self._execute_operations, self._output)

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.resource_group = AAZResourceGroupNameArg()
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        condition_0 = has_value(self.ctx.args.resource_group) and has_value(self.ctx.subscription_id)
        condition_1 = has_value(self.ctx.subscription_id) and has_value(self.ctx.args.resource_group) is not True
        if condition_0:
            self.FleetsListByResourceGroup(ctx=self.ctx)()
        if condition_1:
            self.FleetsListBySubscription(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance.value, client_flatten=True)
        next_link = self.deserialize_output(self.ctx.vars.instance.next_link)
        return result, next_link

    class FleetsListByResourceGroup(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.AzureFleet/fleets",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "MgmtErrorFormat"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2024-11-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()

            _schema_on_200 = cls._schema_on_200
            _schema_on_200.next_link = AAZStrType(
                serialized_name="nextLink",
            )
            _schema_on_200.value = AAZListType(
                flags={"required": True},
            )

            value = cls._schema_on_200.value
            value.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element
            _element.id = AAZStrType(
                flags={"read_only": True},
            )
            _element.identity = AAZIdentityObjectType()
            _element.location = AAZStrType(
                flags={"required": True},
            )
            _element.name = AAZStrType(
                flags={"read_only": True},
            )
            _element.plan = AAZObjectType()
            _element.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            _element.system_data = AAZObjectType(
                serialized_name="systemData",
                flags={"read_only": True},
            )
            _element.tags = AAZDictType()
            _element.type = AAZStrType(
                flags={"read_only": True},
            )
            _element.zones = AAZListType()

            identity = cls._schema_on_200.value.Element.identity
            identity.principal_id = AAZStrType(
                serialized_name="principalId",
                flags={"read_only": True},
            )
            identity.tenant_id = AAZStrType(
                serialized_name="tenantId",
                flags={"read_only": True},
            )
            identity.type = AAZStrType(
                flags={"required": True},
            )
            identity.user_assigned_identities = AAZDictType(
                serialized_name="userAssignedIdentities",
            )

            user_assigned_identities = cls._schema_on_200.value.Element.identity.user_assigned_identities
            user_assigned_identities.Element = AAZObjectType(
                nullable=True,
            )

            _element = cls._schema_on_200.value.Element.identity.user_assigned_identities.Element
            _element.client_id = AAZStrType(
                serialized_name="clientId",
                flags={"read_only": True},
            )
            _element.principal_id = AAZStrType(
                serialized_name="principalId",
                flags={"read_only": True},
            )

            plan = cls._schema_on_200.value.Element.plan
            plan.name = AAZStrType(
                flags={"required": True},
            )
            plan.product = AAZStrType(
                flags={"required": True},
            )
            plan.promotion_code = AAZStrType(
                serialized_name="promotionCode",
            )
            plan.publisher = AAZStrType(
                flags={"required": True},
            )
            plan.version = AAZStrType()

            properties = cls._schema_on_200.value.Element.properties
            properties.additional_locations_profile = AAZObjectType(
                serialized_name="additionalLocationsProfile",
            )
            properties.compute_profile = AAZObjectType(
                serialized_name="computeProfile",
                flags={"required": True},
            )
            properties.provisioning_state = AAZStrType(
                serialized_name="provisioningState",
                flags={"read_only": True},
            )
            properties.regular_priority_profile = AAZObjectType(
                serialized_name="regularPriorityProfile",
            )
            properties.spot_priority_profile = AAZObjectType(
                serialized_name="spotPriorityProfile",
            )
            properties.time_created = AAZStrType(
                serialized_name="timeCreated",
                flags={"read_only": True},
            )
            properties.unique_id = AAZStrType(
                serialized_name="uniqueId",
                flags={"read_only": True},
            )
            properties.vm_attributes = AAZObjectType(
                serialized_name="vmAttributes",
            )
            properties.vm_sizes_profile = AAZListType(
                serialized_name="vmSizesProfile",
                flags={"required": True},
            )

            additional_locations_profile = cls._schema_on_200.value.Element.properties.additional_locations_profile
            additional_locations_profile.location_profiles = AAZListType(
                serialized_name="locationProfiles",
                flags={"required": True},
            )

            location_profiles = cls._schema_on_200.value.Element.properties.additional_locations_profile.location_profiles
            location_profiles.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.additional_locations_profile.location_profiles.Element
            _element.location = AAZStrType(
                flags={"required": True},
            )
            _element.virtual_machine_profile_override = AAZObjectType(
                serialized_name="virtualMachineProfileOverride",
            )
            _ListHelper._build_schema_base_virtual_machine_profile_read(_element.virtual_machine_profile_override)

            compute_profile = cls._schema_on_200.value.Element.properties.compute_profile
            compute_profile.additional_virtual_machine_capabilities = AAZObjectType(
                serialized_name="additionalVirtualMachineCapabilities",
            )
            compute_profile.base_virtual_machine_profile = AAZObjectType(
                serialized_name="baseVirtualMachineProfile",
                flags={"required": True},
            )
            _ListHelper._build_schema_base_virtual_machine_profile_read(compute_profile.base_virtual_machine_profile)
            compute_profile.compute_api_version = AAZStrType(
                serialized_name="computeApiVersion",
            )
            compute_profile.platform_fault_domain_count = AAZIntType(
                serialized_name="platformFaultDomainCount",
            )

            additional_virtual_machine_capabilities = cls._schema_on_200.value.Element.properties.compute_profile.additional_virtual_machine_capabilities
            additional_virtual_machine_capabilities.hibernation_enabled = AAZBoolType(
                serialized_name="hibernationEnabled",
            )
            additional_virtual_machine_capabilities.ultra_ssd_enabled = AAZBoolType(
                serialized_name="ultraSSDEnabled",
            )

            regular_priority_profile = cls._schema_on_200.value.Element.properties.regular_priority_profile
            regular_priority_profile.allocation_strategy = AAZStrType(
                serialized_name="allocationStrategy",
            )
            regular_priority_profile.capacity = AAZIntType()
            regular_priority_profile.min_capacity = AAZIntType(
                serialized_name="minCapacity",
            )

            spot_priority_profile = cls._schema_on_200.value.Element.properties.spot_priority_profile
            spot_priority_profile.allocation_strategy = AAZStrType(
                serialized_name="allocationStrategy",
            )
            spot_priority_profile.capacity = AAZIntType()
            spot_priority_profile.eviction_policy = AAZStrType(
                serialized_name="evictionPolicy",
            )
            spot_priority_profile.maintain = AAZBoolType()
            spot_priority_profile.max_price_per_vm = AAZFloatType(
                serialized_name="maxPricePerVM",
            )
            spot_priority_profile.min_capacity = AAZIntType(
                serialized_name="minCapacity",
            )

            vm_attributes = cls._schema_on_200.value.Element.properties.vm_attributes
            vm_attributes.accelerator_count = AAZObjectType(
                serialized_name="acceleratorCount",
            )
            _ListHelper._build_schema_vm_attribute_min_max_integer_read(vm_attributes.accelerator_count)
            vm_attributes.accelerator_manufacturers = AAZListType(
                serialized_name="acceleratorManufacturers",
            )
            vm_attributes.accelerator_support = AAZStrType(
                serialized_name="acceleratorSupport",
            )
            vm_attributes.accelerator_types = AAZListType(
                serialized_name="acceleratorTypes",
            )
            vm_attributes.architecture_types = AAZListType(
                serialized_name="architectureTypes",
            )
            vm_attributes.burstable_support = AAZStrType(
                serialized_name="burstableSupport",
            )
            vm_attributes.cpu_manufacturers = AAZListType(
                serialized_name="cpuManufacturers",
            )
            vm_attributes.data_disk_count = AAZObjectType(
                serialized_name="dataDiskCount",
            )
            _ListHelper._build_schema_vm_attribute_min_max_integer_read(vm_attributes.data_disk_count)
            vm_attributes.excluded_vm_sizes = AAZListType(
                serialized_name="excludedVMSizes",
            )
            vm_attributes.local_storage_disk_types = AAZListType(
                serialized_name="localStorageDiskTypes",
            )
            vm_attributes.local_storage_in_gi_b = AAZObjectType(
                serialized_name="localStorageInGiB",
            )
            _ListHelper._build_schema_vm_attribute_min_max_double_read(vm_attributes.local_storage_in_gi_b)
            vm_attributes.local_storage_support = AAZStrType(
                serialized_name="localStorageSupport",
            )
            vm_attributes.memory_in_gi_b = AAZObjectType(
                serialized_name="memoryInGiB",
                flags={"required": True},
            )
            _ListHelper._build_schema_vm_attribute_min_max_double_read(vm_attributes.memory_in_gi_b)
            vm_attributes.memory_in_gi_b_per_v_cpu = AAZObjectType(
                serialized_name="memoryInGiBPerVCpu",
            )
            _ListHelper._build_schema_vm_attribute_min_max_double_read(vm_attributes.memory_in_gi_b_per_v_cpu)
            vm_attributes.network_bandwidth_in_mbps = AAZObjectType(
                serialized_name="networkBandwidthInMbps",
            )
            _ListHelper._build_schema_vm_attribute_min_max_double_read(vm_attributes.network_bandwidth_in_mbps)
            vm_attributes.network_interface_count = AAZObjectType(
                serialized_name="networkInterfaceCount",
            )
            _ListHelper._build_schema_vm_attribute_min_max_integer_read(vm_attributes.network_interface_count)
            vm_attributes.rdma_network_interface_count = AAZObjectType(
                serialized_name="rdmaNetworkInterfaceCount",
            )
            _ListHelper._build_schema_vm_attribute_min_max_integer_read(vm_attributes.rdma_network_interface_count)
            vm_attributes.rdma_support = AAZStrType(
                serialized_name="rdmaSupport",
            )
            vm_attributes.v_cpu_count = AAZObjectType(
                serialized_name="vCpuCount",
                flags={"required": True},
            )
            _ListHelper._build_schema_vm_attribute_min_max_integer_read(vm_attributes.v_cpu_count)
            vm_attributes.vm_categories = AAZListType(
                serialized_name="vmCategories",
            )

            accelerator_manufacturers = cls._schema_on_200.value.Element.properties.vm_attributes.accelerator_manufacturers
            accelerator_manufacturers.Element = AAZStrType()

            accelerator_types = cls._schema_on_200.value.Element.properties.vm_attributes.accelerator_types
            accelerator_types.Element = AAZStrType()

            architecture_types = cls._schema_on_200.value.Element.properties.vm_attributes.architecture_types
            architecture_types.Element = AAZStrType()

            cpu_manufacturers = cls._schema_on_200.value.Element.properties.vm_attributes.cpu_manufacturers
            cpu_manufacturers.Element = AAZStrType()

            excluded_vm_sizes = cls._schema_on_200.value.Element.properties.vm_attributes.excluded_vm_sizes
            excluded_vm_sizes.Element = AAZStrType()

            local_storage_disk_types = cls._schema_on_200.value.Element.properties.vm_attributes.local_storage_disk_types
            local_storage_disk_types.Element = AAZStrType()

            vm_categories = cls._schema_on_200.value.Element.properties.vm_attributes.vm_categories
            vm_categories.Element = AAZStrType()

            vm_sizes_profile = cls._schema_on_200.value.Element.properties.vm_sizes_profile
            vm_sizes_profile.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.vm_sizes_profile.Element
            _element.name = AAZStrType(
                flags={"required": True},
            )
            _element.rank = AAZIntType()

            system_data = cls._schema_on_200.value.Element.system_data
            system_data.created_at = AAZStrType(
                serialized_name="createdAt",
            )
            system_data.created_by = AAZStrType(
                serialized_name="createdBy",
            )
            system_data.created_by_type = AAZStrType(
                serialized_name="createdByType",
            )
            system_data.last_modified_at = AAZStrType(
                serialized_name="lastModifiedAt",
            )
            system_data.last_modified_by = AAZStrType(
                serialized_name="lastModifiedBy",
            )
            system_data.last_modified_by_type = AAZStrType(
                serialized_name="lastModifiedByType",
            )

            tags = cls._schema_on_200.value.Element.tags
            tags.Element = AAZStrType()

            zones = cls._schema_on_200.value.Element.zones
            zones.Element = AAZStrType()

            return cls._schema_on_200

    class FleetsListBySubscription(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/providers/Microsoft.AzureFleet/fleets",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "MgmtErrorFormat"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2024-11-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()

            _schema_on_200 = cls._schema_on_200
            _schema_on_200.next_link = AAZStrType(
                serialized_name="nextLink",
            )
            _schema_on_200.value = AAZListType(
                flags={"required": True},
            )

            value = cls._schema_on_200.value
            value.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element
            _element.id = AAZStrType(
                flags={"read_only": True},
            )
            _element.identity = AAZIdentityObjectType()
            _element.location = AAZStrType(
                flags={"required": True},
            )
            _element.name = AAZStrType(
                flags={"read_only": True},
            )
            _element.plan = AAZObjectType()
            _element.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            _element.system_data = AAZObjectType(
                serialized_name="systemData",
                flags={"read_only": True},
            )
            _element.tags = AAZDictType()
            _element.type = AAZStrType(
                flags={"read_only": True},
            )
            _element.zones = AAZListType()

            identity = cls._schema_on_200.value.Element.identity
            identity.principal_id = AAZStrType(
                serialized_name="principalId",
                flags={"read_only": True},
            )
            identity.tenant_id = AAZStrType(
                serialized_name="tenantId",
                flags={"read_only": True},
            )
            identity.type = AAZStrType(
                flags={"required": True},
            )
            identity.user_assigned_identities = AAZDictType(
                serialized_name="userAssignedIdentities",
            )

            user_assigned_identities = cls._schema_on_200.value.Element.identity.user_assigned_identities
            user_assigned_identities.Element = AAZObjectType(
                nullable=True,
            )

            _element = cls._schema_on_200.value.Element.identity.user_assigned_identities.Element
            _element.client_id = AAZStrType(
                serialized_name="clientId",
                flags={"read_only": True},
            )
            _element.principal_id = AAZStrType(
                serialized_name="principalId",
                flags={"read_only": True},
            )

            plan = cls._schema_on_200.value.Element.plan
            plan.name = AAZStrType(
                flags={"required": True},
            )
            plan.product = AAZStrType(
                flags={"required": True},
            )
            plan.promotion_code = AAZStrType(
                serialized_name="promotionCode",
            )
            plan.publisher = AAZStrType(
                flags={"required": True},
            )
            plan.version = AAZStrType()

            properties = cls._schema_on_200.value.Element.properties
            properties.additional_locations_profile = AAZObjectType(
                serialized_name="additionalLocationsProfile",
            )
            properties.compute_profile = AAZObjectType(
                serialized_name="computeProfile",
                flags={"required": True},
            )
            properties.provisioning_state = AAZStrType(
                serialized_name="provisioningState",
                flags={"read_only": True},
            )
            properties.regular_priority_profile = AAZObjectType(
                serialized_name="regularPriorityProfile",
            )
            properties.spot_priority_profile = AAZObjectType(
                serialized_name="spotPriorityProfile",
            )
            properties.time_created = AAZStrType(
                serialized_name="timeCreated",
                flags={"read_only": True},
            )
            properties.unique_id = AAZStrType(
                serialized_name="uniqueId",
                flags={"read_only": True},
            )
            properties.vm_attributes = AAZObjectType(
                serialized_name="vmAttributes",
            )
            properties.vm_sizes_profile = AAZListType(
                serialized_name="vmSizesProfile",
                flags={"required": True},
            )

            additional_locations_profile = cls._schema_on_200.value.Element.properties.additional_locations_profile
            additional_locations_profile.location_profiles = AAZListType(
                serialized_name="locationProfiles",
                flags={"required": True},
            )

            location_profiles = cls._schema_on_200.value.Element.properties.additional_locations_profile.location_profiles
            location_profiles.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.additional_locations_profile.location_profiles.Element
            _element.location = AAZStrType(
                flags={"required": True},
            )
            _element.virtual_machine_profile_override = AAZObjectType(
                serialized_name="virtualMachineProfileOverride",
            )
            _ListHelper._build_schema_base_virtual_machine_profile_read(_element.virtual_machine_profile_override)

            compute_profile = cls._schema_on_200.value.Element.properties.compute_profile
            compute_profile.additional_virtual_machine_capabilities = AAZObjectType(
                serialized_name="additionalVirtualMachineCapabilities",
            )
            compute_profile.base_virtual_machine_profile = AAZObjectType(
                serialized_name="baseVirtualMachineProfile",
                flags={"required": True},
            )
            _ListHelper._build_schema_base_virtual_machine_profile_read(compute_profile.base_virtual_machine_profile)
            compute_profile.compute_api_version = AAZStrType(
                serialized_name="computeApiVersion",
            )
            compute_profile.platform_fault_domain_count = AAZIntType(
                serialized_name="platformFaultDomainCount",
            )

            additional_virtual_machine_capabilities = cls._schema_on_200.value.Element.properties.compute_profile.additional_virtual_machine_capabilities
            additional_virtual_machine_capabilities.hibernation_enabled = AAZBoolType(
                serialized_name="hibernationEnabled",
            )
            additional_virtual_machine_capabilities.ultra_ssd_enabled = AAZBoolType(
                serialized_name="ultraSSDEnabled",
            )

            regular_priority_profile = cls._schema_on_200.value.Element.properties.regular_priority_profile
            regular_priority_profile.allocation_strategy = AAZStrType(
                serialized_name="allocationStrategy",
            )
            regular_priority_profile.capacity = AAZIntType()
            regular_priority_profile.min_capacity = AAZIntType(
                serialized_name="minCapacity",
            )

            spot_priority_profile = cls._schema_on_200.value.Element.properties.spot_priority_profile
            spot_priority_profile.allocation_strategy = AAZStrType(
                serialized_name="allocationStrategy",
            )
            spot_priority_profile.capacity = AAZIntType()
            spot_priority_profile.eviction_policy = AAZStrType(
                serialized_name="evictionPolicy",
            )
            spot_priority_profile.maintain = AAZBoolType()
            spot_priority_profile.max_price_per_vm = AAZFloatType(
                serialized_name="maxPricePerVM",
            )
            spot_priority_profile.min_capacity = AAZIntType(
                serialized_name="minCapacity",
            )

            vm_attributes = cls._schema_on_200.value.Element.properties.vm_attributes
            vm_attributes.accelerator_count = AAZObjectType(
                serialized_name="acceleratorCount",
            )
            _ListHelper._build_schema_vm_attribute_min_max_integer_read(vm_attributes.accelerator_count)
            vm_attributes.accelerator_manufacturers = AAZListType(
                serialized_name="acceleratorManufacturers",
            )
            vm_attributes.accelerator_support = AAZStrType(
                serialized_name="acceleratorSupport",
            )
            vm_attributes.accelerator_types = AAZListType(
                serialized_name="acceleratorTypes",
            )
            vm_attributes.architecture_types = AAZListType(
                serialized_name="architectureTypes",
            )
            vm_attributes.burstable_support = AAZStrType(
                serialized_name="burstableSupport",
            )
            vm_attributes.cpu_manufacturers = AAZListType(
                serialized_name="cpuManufacturers",
            )
            vm_attributes.data_disk_count = AAZObjectType(
                serialized_name="dataDiskCount",
            )
            _ListHelper._build_schema_vm_attribute_min_max_integer_read(vm_attributes.data_disk_count)
            vm_attributes.excluded_vm_sizes = AAZListType(
                serialized_name="excludedVMSizes",
            )
            vm_attributes.local_storage_disk_types = AAZListType(
                serialized_name="localStorageDiskTypes",
            )
            vm_attributes.local_storage_in_gi_b = AAZObjectType(
                serialized_name="localStorageInGiB",
            )
            _ListHelper._build_schema_vm_attribute_min_max_double_read(vm_attributes.local_storage_in_gi_b)
            vm_attributes.local_storage_support = AAZStrType(
                serialized_name="localStorageSupport",
            )
            vm_attributes.memory_in_gi_b = AAZObjectType(
                serialized_name="memoryInGiB",
                flags={"required": True},
            )
            _ListHelper._build_schema_vm_attribute_min_max_double_read(vm_attributes.memory_in_gi_b)
            vm_attributes.memory_in_gi_b_per_v_cpu = AAZObjectType(
                serialized_name="memoryInGiBPerVCpu",
            )
            _ListHelper._build_schema_vm_attribute_min_max_double_read(vm_attributes.memory_in_gi_b_per_v_cpu)
            vm_attributes.network_bandwidth_in_mbps = AAZObjectType(
                serialized_name="networkBandwidthInMbps",
            )
            _ListHelper._build_schema_vm_attribute_min_max_double_read(vm_attributes.network_bandwidth_in_mbps)
            vm_attributes.network_interface_count = AAZObjectType(
                serialized_name="networkInterfaceCount",
            )
            _ListHelper._build_schema_vm_attribute_min_max_integer_read(vm_attributes.network_interface_count)
            vm_attributes.rdma_network_interface_count = AAZObjectType(
                serialized_name="rdmaNetworkInterfaceCount",
            )
            _ListHelper._build_schema_vm_attribute_min_max_integer_read(vm_attributes.rdma_network_interface_count)
            vm_attributes.rdma_support = AAZStrType(
                serialized_name="rdmaSupport",
            )
            vm_attributes.v_cpu_count = AAZObjectType(
                serialized_name="vCpuCount",
                flags={"required": True},
            )
            _ListHelper._build_schema_vm_attribute_min_max_integer_read(vm_attributes.v_cpu_count)
            vm_attributes.vm_categories = AAZListType(
                serialized_name="vmCategories",
            )

            accelerator_manufacturers = cls._schema_on_200.value.Element.properties.vm_attributes.accelerator_manufacturers
            accelerator_manufacturers.Element = AAZStrType()

            accelerator_types = cls._schema_on_200.value.Element.properties.vm_attributes.accelerator_types
            accelerator_types.Element = AAZStrType()

            architecture_types = cls._schema_on_200.value.Element.properties.vm_attributes.architecture_types
            architecture_types.Element = AAZStrType()

            cpu_manufacturers = cls._schema_on_200.value.Element.properties.vm_attributes.cpu_manufacturers
            cpu_manufacturers.Element = AAZStrType()

            excluded_vm_sizes = cls._schema_on_200.value.Element.properties.vm_attributes.excluded_vm_sizes
            excluded_vm_sizes.Element = AAZStrType()

            local_storage_disk_types = cls._schema_on_200.value.Element.properties.vm_attributes.local_storage_disk_types
            local_storage_disk_types.Element = AAZStrType()

            vm_categories = cls._schema_on_200.value.Element.properties.vm_attributes.vm_categories
            vm_categories.Element = AAZStrType()

            vm_sizes_profile = cls._schema_on_200.value.Element.properties.vm_sizes_profile
            vm_sizes_profile.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.vm_sizes_profile.Element
            _element.name = AAZStrType(
                flags={"required": True},
            )
            _element.rank = AAZIntType()

            system_data = cls._schema_on_200.value.Element.system_data
            system_data.created_at = AAZStrType(
                serialized_name="createdAt",
            )
            system_data.created_by = AAZStrType(
                serialized_name="createdBy",
            )
            system_data.created_by_type = AAZStrType(
                serialized_name="createdByType",
            )
            system_data.last_modified_at = AAZStrType(
                serialized_name="lastModifiedAt",
            )
            system_data.last_modified_by = AAZStrType(
                serialized_name="lastModifiedBy",
            )
            system_data.last_modified_by_type = AAZStrType(
                serialized_name="lastModifiedByType",
            )

            tags = cls._schema_on_200.value.Element.tags
            tags.Element = AAZStrType()

            zones = cls._schema_on_200.value.Element.zones
            zones.Element = AAZStrType()

            return cls._schema_on_200


class _ListHelper:
    """Helper class for List"""

    _schema_api_entity_reference_read = None

    @classmethod
    def _build_schema_api_entity_reference_read(cls, _schema):
        if cls._schema_api_entity_reference_read is not None:
            _schema.id = cls._schema_api_entity_reference_read.id
            return

        cls._schema_api_entity_reference_read = _schema_api_entity_reference_read = AAZObjectType()

        api_entity_reference_read = _schema_api_entity_reference_read
        api_entity_reference_read.id = AAZStrType()

        _schema.id = cls._schema_api_entity_reference_read.id

    _schema_base_virtual_machine_profile_read = None

    @classmethod
    def _build_schema_base_virtual_machine_profile_read(cls, _schema):
        if cls._schema_base_virtual_machine_profile_read is not None:
            _schema.application_profile = cls._schema_base_virtual_machine_profile_read.application_profile
            _schema.capacity_reservation = cls._schema_base_virtual_machine_profile_read.capacity_reservation
            _schema.diagnostics_profile = cls._schema_base_virtual_machine_profile_read.diagnostics_profile
            _schema.extension_profile = cls._schema_base_virtual_machine_profile_read.extension_profile
            _schema.hardware_profile = cls._schema_base_virtual_machine_profile_read.hardware_profile
            _schema.license_type = cls._schema_base_virtual_machine_profile_read.license_type
            _schema.network_profile = cls._schema_base_virtual_machine_profile_read.network_profile
            _schema.os_profile = cls._schema_base_virtual_machine_profile_read.os_profile
            _schema.scheduled_events_profile = cls._schema_base_virtual_machine_profile_read.scheduled_events_profile
            _schema.security_posture_reference = cls._schema_base_virtual_machine_profile_read.security_posture_reference
            _schema.security_profile = cls._schema_base_virtual_machine_profile_read.security_profile
            _schema.service_artifact_reference = cls._schema_base_virtual_machine_profile_read.service_artifact_reference
            _schema.storage_profile = cls._schema_base_virtual_machine_profile_read.storage_profile
            _schema.time_created = cls._schema_base_virtual_machine_profile_read.time_created
            _schema.user_data = cls._schema_base_virtual_machine_profile_read.user_data
            return

        cls._schema_base_virtual_machine_profile_read = _schema_base_virtual_machine_profile_read = AAZObjectType()

        base_virtual_machine_profile_read = _schema_base_virtual_machine_profile_read
        base_virtual_machine_profile_read.application_profile = AAZObjectType(
            serialized_name="applicationProfile",
        )
        base_virtual_machine_profile_read.capacity_reservation = AAZObjectType(
            serialized_name="capacityReservation",
        )
        base_virtual_machine_profile_read.diagnostics_profile = AAZObjectType(
            serialized_name="diagnosticsProfile",
        )
        base_virtual_machine_profile_read.extension_profile = AAZObjectType(
            serialized_name="extensionProfile",
        )
        base_virtual_machine_profile_read.hardware_profile = AAZObjectType(
            serialized_name="hardwareProfile",
        )
        base_virtual_machine_profile_read.license_type = AAZStrType(
            serialized_name="licenseType",
        )
        base_virtual_machine_profile_read.network_profile = AAZObjectType(
            serialized_name="networkProfile",
        )
        base_virtual_machine_profile_read.os_profile = AAZObjectType(
            serialized_name="osProfile",
        )
        base_virtual_machine_profile_read.scheduled_events_profile = AAZObjectType(
            serialized_name="scheduledEventsProfile",
        )
        base_virtual_machine_profile_read.security_posture_reference = AAZObjectType(
            serialized_name="securityPostureReference",
        )
        base_virtual_machine_profile_read.security_profile = AAZObjectType(
            serialized_name="securityProfile",
        )
        base_virtual_machine_profile_read.service_artifact_reference = AAZObjectType(
            serialized_name="serviceArtifactReference",
        )
        base_virtual_machine_profile_read.storage_profile = AAZObjectType(
            serialized_name="storageProfile",
        )
        base_virtual_machine_profile_read.time_created = AAZStrType(
            serialized_name="timeCreated",
            flags={"read_only": True},
        )
        base_virtual_machine_profile_read.user_data = AAZStrType(
            serialized_name="userData",
        )

        application_profile = _schema_base_virtual_machine_profile_read.application_profile
        application_profile.gallery_applications = AAZListType(
            serialized_name="galleryApplications",
        )

        gallery_applications = _schema_base_virtual_machine_profile_read.application_profile.gallery_applications
        gallery_applications.Element = AAZObjectType()

        _element = _schema_base_virtual_machine_profile_read.application_profile.gallery_applications.Element
        _element.configuration_reference = AAZStrType(
            serialized_name="configurationReference",
        )
        _element.enable_automatic_upgrade = AAZBoolType(
            serialized_name="enableAutomaticUpgrade",
        )
        _element.order = AAZIntType()
        _element.package_reference_id = AAZStrType(
            serialized_name="packageReferenceId",
            flags={"required": True},
        )
        _element.tags = AAZStrType()
        _element.treat_failure_as_deployment_failure = AAZBoolType(
            serialized_name="treatFailureAsDeploymentFailure",
        )

        capacity_reservation = _schema_base_virtual_machine_profile_read.capacity_reservation
        capacity_reservation.capacity_reservation_group = AAZObjectType(
            serialized_name="capacityReservationGroup",
        )
        cls._build_schema_sub_resource_read(capacity_reservation.capacity_reservation_group)

        diagnostics_profile = _schema_base_virtual_machine_profile_read.diagnostics_profile
        diagnostics_profile.boot_diagnostics = AAZObjectType(
            serialized_name="bootDiagnostics",
        )

        boot_diagnostics = _schema_base_virtual_machine_profile_read.diagnostics_profile.boot_diagnostics
        boot_diagnostics.enabled = AAZBoolType()
        boot_diagnostics.storage_uri = AAZStrType(
            serialized_name="storageUri",
        )

        extension_profile = _schema_base_virtual_machine_profile_read.extension_profile
        extension_profile.extensions = AAZListType()
        extension_profile.extensions_time_budget = AAZStrType(
            serialized_name="extensionsTimeBudget",
        )

        extensions = _schema_base_virtual_machine_profile_read.extension_profile.extensions
        extensions.Element = AAZObjectType()

        _element = _schema_base_virtual_machine_profile_read.extension_profile.extensions.Element
        _element.id = AAZStrType(
            flags={"read_only": True},
        )
        _element.name = AAZStrType()
        _element.properties = AAZObjectType()
        _element.type = AAZStrType(
            flags={"read_only": True},
        )

        properties = _schema_base_virtual_machine_profile_read.extension_profile.extensions.Element.properties
        properties.auto_upgrade_minor_version = AAZBoolType(
            serialized_name="autoUpgradeMinorVersion",
        )
        properties.enable_automatic_upgrade = AAZBoolType(
            serialized_name="enableAutomaticUpgrade",
        )
        properties.force_update_tag = AAZStrType(
            serialized_name="forceUpdateTag",
        )
        properties.protected_settings_from_key_vault = AAZObjectType(
            serialized_name="protectedSettingsFromKeyVault",
        )
        properties.provision_after_extensions = AAZListType(
            serialized_name="provisionAfterExtensions",
        )
        properties.provisioning_state = AAZStrType(
            serialized_name="provisioningState",
            flags={"read_only": True},
        )
        properties.publisher = AAZStrType()
        properties.settings = AAZFreeFormDictType()
        properties.suppress_failures = AAZBoolType(
            serialized_name="suppressFailures",
        )
        properties.type = AAZStrType()
        properties.type_handler_version = AAZStrType(
            serialized_name="typeHandlerVersion",
        )

        protected_settings_from_key_vault = _schema_base_virtual_machine_profile_read.extension_profile.extensions.Element.properties.protected_settings_from_key_vault
        protected_settings_from_key_vault.secret_url = AAZStrType(
            serialized_name="secretUrl",
            flags={"required": True},
        )
        protected_settings_from_key_vault.source_vault = AAZObjectType(
            serialized_name="sourceVault",
            flags={"required": True},
        )
        cls._build_schema_sub_resource_read(protected_settings_from_key_vault.source_vault)

        provision_after_extensions = _schema_base_virtual_machine_profile_read.extension_profile.extensions.Element.properties.provision_after_extensions
        provision_after_extensions.Element = AAZStrType()

        hardware_profile = _schema_base_virtual_machine_profile_read.hardware_profile
        hardware_profile.vm_size_properties = AAZObjectType(
            serialized_name="vmSizeProperties",
        )

        vm_size_properties = _schema_base_virtual_machine_profile_read.hardware_profile.vm_size_properties
        vm_size_properties.v_cp_us_available = AAZIntType(
            serialized_name="vCPUsAvailable",
        )
        vm_size_properties.v_cp_us_per_core = AAZIntType(
            serialized_name="vCPUsPerCore",
        )

        network_profile = _schema_base_virtual_machine_profile_read.network_profile
        network_profile.health_probe = AAZObjectType(
            serialized_name="healthProbe",
        )
        cls._build_schema_api_entity_reference_read(network_profile.health_probe)
        network_profile.network_api_version = AAZStrType(
            serialized_name="networkApiVersion",
        )
        network_profile.network_interface_configurations = AAZListType(
            serialized_name="networkInterfaceConfigurations",
        )

        network_interface_configurations = _schema_base_virtual_machine_profile_read.network_profile.network_interface_configurations
        network_interface_configurations.Element = AAZObjectType()

        _element = _schema_base_virtual_machine_profile_read.network_profile.network_interface_configurations.Element
        _element.name = AAZStrType(
            flags={"required": True},
        )
        _element.properties = AAZObjectType()

        properties = _schema_base_virtual_machine_profile_read.network_profile.network_interface_configurations.Element.properties
        properties.auxiliary_mode = AAZStrType(
            serialized_name="auxiliaryMode",
        )
        properties.auxiliary_sku = AAZStrType(
            serialized_name="auxiliarySku",
        )
        properties.delete_option = AAZStrType(
            serialized_name="deleteOption",
        )
        properties.disable_tcp_state_tracking = AAZBoolType(
            serialized_name="disableTcpStateTracking",
        )
        properties.dns_settings = AAZObjectType(
            serialized_name="dnsSettings",
        )
        properties.enable_accelerated_networking = AAZBoolType(
            serialized_name="enableAcceleratedNetworking",
        )
        properties.enable_fpga = AAZBoolType(
            serialized_name="enableFpga",
        )
        properties.enable_ip_forwarding = AAZBoolType(
            serialized_name="enableIPForwarding",
        )
        properties.ip_configurations = AAZListType(
            serialized_name="ipConfigurations",
            flags={"required": True},
        )
        properties.network_security_group = AAZObjectType(
            serialized_name="networkSecurityGroup",
        )
        cls._build_schema_sub_resource_read(properties.network_security_group)
        properties.primary = AAZBoolType()

        dns_settings = _schema_base_virtual_machine_profile_read.network_profile.network_interface_configurations.Element.properties.dns_settings
        dns_settings.dns_servers = AAZListType(
            serialized_name="dnsServers",
        )

        dns_servers = _schema_base_virtual_machine_profile_read.network_profile.network_interface_configurations.Element.properties.dns_settings.dns_servers
        dns_servers.Element = AAZStrType()

        ip_configurations = _schema_base_virtual_machine_profile_read.network_profile.network_interface_configurations.Element.properties.ip_configurations
        ip_configurations.Element = AAZObjectType()

        _element = _schema_base_virtual_machine_profile_read.network_profile.network_interface_configurations.Element.properties.ip_configurations.Element
        _element.name = AAZStrType(
            flags={"required": True},
        )
        _element.properties = AAZObjectType()

        properties = _schema_base_virtual_machine_profile_read.network_profile.network_interface_configurations.Element.properties.ip_configurations.Element.properties
        properties.application_gateway_backend_address_pools = AAZListType(
            serialized_name="applicationGatewayBackendAddressPools",
        )
        properties.application_security_groups = AAZListType(
            serialized_name="applicationSecurityGroups",
        )
        properties.load_balancer_backend_address_pools = AAZListType(
            serialized_name="loadBalancerBackendAddressPools",
        )
        properties.load_balancer_inbound_nat_pools = AAZListType(
            serialized_name="loadBalancerInboundNatPools",
        )
        properties.primary = AAZBoolType()
        properties.private_ip_address_version = AAZStrType(
            serialized_name="privateIPAddressVersion",
        )
        properties.public_ip_address_configuration = AAZObjectType(
            serialized_name="publicIPAddressConfiguration",
        )
        properties.subnet = AAZObjectType()
        cls._build_schema_api_entity_reference_read(properties.subnet)

        application_gateway_backend_address_pools = _schema_base_virtual_machine_profile_read.network_profile.network_interface_configurations.Element.properties.ip_configurations.Element.properties.application_gateway_backend_address_pools
        application_gateway_backend_address_pools.Element = AAZObjectType()
        cls._build_schema_sub_resource_read(application_gateway_backend_address_pools.Element)

        application_security_groups = _schema_base_virtual_machine_profile_read.network_profile.network_interface_configurations.Element.properties.ip_configurations.Element.properties.application_security_groups
        application_security_groups.Element = AAZObjectType()
        cls._build_schema_sub_resource_read(application_security_groups.Element)

        load_balancer_backend_address_pools = _schema_base_virtual_machine_profile_read.network_profile.network_interface_configurations.Element.properties.ip_configurations.Element.properties.load_balancer_backend_address_pools
        load_balancer_backend_address_pools.Element = AAZObjectType()
        cls._build_schema_sub_resource_read(load_balancer_backend_address_pools.Element)

        load_balancer_inbound_nat_pools = _schema_base_virtual_machine_profile_read.network_profile.network_interface_configurations.Element.properties.ip_configurations.Element.properties.load_balancer_inbound_nat_pools
        load_balancer_inbound_nat_pools.Element = AAZObjectType()
        cls._build_schema_sub_resource_read(load_balancer_inbound_nat_pools.Element)

        public_ip_address_configuration = _schema_base_virtual_machine_profile_read.network_profile.network_interface_configurations.Element.properties.ip_configurations.Element.properties.public_ip_address_configuration
        public_ip_address_configuration.name = AAZStrType(
            flags={"required": True},
        )
        public_ip_address_configuration.properties = AAZObjectType()
        public_ip_address_configuration.sku = AAZObjectType()

        properties = _schema_base_virtual_machine_profile_read.network_profile.network_interface_configurations.Element.properties.ip_configurations.Element.properties.public_ip_address_configuration.properties
        properties.delete_option = AAZStrType(
            serialized_name="deleteOption",
        )
        properties.dns_settings = AAZObjectType(
            serialized_name="dnsSettings",
        )
        properties.idle_timeout_in_minutes = AAZIntType(
            serialized_name="idleTimeoutInMinutes",
        )
        properties.ip_tags = AAZListType(
            serialized_name="ipTags",
        )
        properties.public_ip_address_version = AAZStrType(
            serialized_name="publicIPAddressVersion",
        )
        properties.public_ip_prefix = AAZObjectType(
            serialized_name="publicIPPrefix",
        )
        cls._build_schema_sub_resource_read(properties.public_ip_prefix)

        dns_settings = _schema_base_virtual_machine_profile_read.network_profile.network_interface_configurations.Element.properties.ip_configurations.Element.properties.public_ip_address_configuration.properties.dns_settings
        dns_settings.domain_name_label = AAZStrType(
            serialized_name="domainNameLabel",
            flags={"required": True},
        )
        dns_settings.domain_name_label_scope = AAZStrType(
            serialized_name="domainNameLabelScope",
        )

        ip_tags = _schema_base_virtual_machine_profile_read.network_profile.network_interface_configurations.Element.properties.ip_configurations.Element.properties.public_ip_address_configuration.properties.ip_tags
        ip_tags.Element = AAZObjectType()

        _element = _schema_base_virtual_machine_profile_read.network_profile.network_interface_configurations.Element.properties.ip_configurations.Element.properties.public_ip_address_configuration.properties.ip_tags.Element
        _element.ip_tag_type = AAZStrType(
            serialized_name="ipTagType",
        )
        _element.tag = AAZStrType()

        sku = _schema_base_virtual_machine_profile_read.network_profile.network_interface_configurations.Element.properties.ip_configurations.Element.properties.public_ip_address_configuration.sku
        sku.name = AAZStrType()
        sku.tier = AAZStrType()

        os_profile = _schema_base_virtual_machine_profile_read.os_profile
        os_profile.admin_username = AAZStrType(
            serialized_name="adminUsername",
        )
        os_profile.allow_extension_operations = AAZBoolType(
            serialized_name="allowExtensionOperations",
        )
        os_profile.computer_name_prefix = AAZStrType(
            serialized_name="computerNamePrefix",
        )
        os_profile.linux_configuration = AAZObjectType(
            serialized_name="linuxConfiguration",
        )
        os_profile.require_guest_provision_signal = AAZBoolType(
            serialized_name="requireGuestProvisionSignal",
        )
        os_profile.secrets = AAZListType()
        os_profile.windows_configuration = AAZObjectType(
            serialized_name="windowsConfiguration",
        )

        linux_configuration = _schema_base_virtual_machine_profile_read.os_profile.linux_configuration
        linux_configuration.disable_password_authentication = AAZBoolType(
            serialized_name="disablePasswordAuthentication",
        )
        linux_configuration.enable_vm_agent_platform_updates = AAZBoolType(
            serialized_name="enableVMAgentPlatformUpdates",
        )
        linux_configuration.patch_settings = AAZObjectType(
            serialized_name="patchSettings",
        )
        linux_configuration.provision_vm_agent = AAZBoolType(
            serialized_name="provisionVMAgent",
        )
        linux_configuration.ssh = AAZObjectType()

        patch_settings = _schema_base_virtual_machine_profile_read.os_profile.linux_configuration.patch_settings
        patch_settings.assessment_mode = AAZStrType(
            serialized_name="assessmentMode",
        )
        patch_settings.automatic_by_platform_settings = AAZObjectType(
            serialized_name="automaticByPlatformSettings",
        )
        patch_settings.patch_mode = AAZStrType(
            serialized_name="patchMode",
        )

        automatic_by_platform_settings = _schema_base_virtual_machine_profile_read.os_profile.linux_configuration.patch_settings.automatic_by_platform_settings
        automatic_by_platform_settings.bypass_platform_safety_checks_on_user_schedule = AAZBoolType(
            serialized_name="bypassPlatformSafetyChecksOnUserSchedule",
        )
        automatic_by_platform_settings.reboot_setting = AAZStrType(
            serialized_name="rebootSetting",
        )

        ssh = _schema_base_virtual_machine_profile_read.os_profile.linux_configuration.ssh
        ssh.public_keys = AAZListType(
            serialized_name="publicKeys",
        )

        public_keys = _schema_base_virtual_machine_profile_read.os_profile.linux_configuration.ssh.public_keys
        public_keys.Element = AAZObjectType()

        _element = _schema_base_virtual_machine_profile_read.os_profile.linux_configuration.ssh.public_keys.Element
        _element.key_data = AAZStrType(
            serialized_name="keyData",
        )
        _element.path = AAZStrType()

        secrets = _schema_base_virtual_machine_profile_read.os_profile.secrets
        secrets.Element = AAZObjectType()

        _element = _schema_base_virtual_machine_profile_read.os_profile.secrets.Element
        _element.source_vault = AAZObjectType(
            serialized_name="sourceVault",
        )
        cls._build_schema_sub_resource_read(_element.source_vault)
        _element.vault_certificates = AAZListType(
            serialized_name="vaultCertificates",
        )

        vault_certificates = _schema_base_virtual_machine_profile_read.os_profile.secrets.Element.vault_certificates
        vault_certificates.Element = AAZObjectType()

        _element = _schema_base_virtual_machine_profile_read.os_profile.secrets.Element.vault_certificates.Element
        _element.certificate_store = AAZStrType(
            serialized_name="certificateStore",
        )
        _element.certificate_url = AAZStrType(
            serialized_name="certificateUrl",
        )

        windows_configuration = _schema_base_virtual_machine_profile_read.os_profile.windows_configuration
        windows_configuration.additional_unattend_content = AAZListType(
            serialized_name="additionalUnattendContent",
        )
        windows_configuration.enable_automatic_updates = AAZBoolType(
            serialized_name="enableAutomaticUpdates",
        )
        windows_configuration.enable_vm_agent_platform_updates = AAZBoolType(
            serialized_name="enableVMAgentPlatformUpdates",
        )
        windows_configuration.patch_settings = AAZObjectType(
            serialized_name="patchSettings",
        )
        windows_configuration.provision_vm_agent = AAZBoolType(
            serialized_name="provisionVMAgent",
        )
        windows_configuration.time_zone = AAZStrType(
            serialized_name="timeZone",
        )
        windows_configuration.win_rm = AAZObjectType(
            serialized_name="winRM",
        )

        additional_unattend_content = _schema_base_virtual_machine_profile_read.os_profile.windows_configuration.additional_unattend_content
        additional_unattend_content.Element = AAZObjectType()

        _element = _schema_base_virtual_machine_profile_read.os_profile.windows_configuration.additional_unattend_content.Element
        _element.component_name = AAZStrType(
            serialized_name="componentName",
        )
        _element.pass_name = AAZStrType(
            serialized_name="passName",
        )
        _element.setting_name = AAZStrType(
            serialized_name="settingName",
        )

        patch_settings = _schema_base_virtual_machine_profile_read.os_profile.windows_configuration.patch_settings
        patch_settings.assessment_mode = AAZStrType(
            serialized_name="assessmentMode",
        )
        patch_settings.automatic_by_platform_settings = AAZObjectType(
            serialized_name="automaticByPlatformSettings",
        )
        patch_settings.enable_hotpatching = AAZBoolType(
            serialized_name="enableHotpatching",
        )
        patch_settings.patch_mode = AAZStrType(
            serialized_name="patchMode",
        )

        automatic_by_platform_settings = _schema_base_virtual_machine_profile_read.os_profile.windows_configuration.patch_settings.automatic_by_platform_settings
        automatic_by_platform_settings.bypass_platform_safety_checks_on_user_schedule = AAZBoolType(
            serialized_name="bypassPlatformSafetyChecksOnUserSchedule",
        )
        automatic_by_platform_settings.reboot_setting = AAZStrType(
            serialized_name="rebootSetting",
        )

        win_rm = _schema_base_virtual_machine_profile_read.os_profile.windows_configuration.win_rm
        win_rm.listeners = AAZListType()

        listeners = _schema_base_virtual_machine_profile_read.os_profile.windows_configuration.win_rm.listeners
        listeners.Element = AAZObjectType()

        _element = _schema_base_virtual_machine_profile_read.os_profile.windows_configuration.win_rm.listeners.Element
        _element.certificate_url = AAZStrType(
            serialized_name="certificateUrl",
        )
        _element.protocol = AAZStrType()

        scheduled_events_profile = _schema_base_virtual_machine_profile_read.scheduled_events_profile
        scheduled_events_profile.os_image_notification_profile = AAZObjectType(
            serialized_name="osImageNotificationProfile",
        )
        scheduled_events_profile.terminate_notification_profile = AAZObjectType(
            serialized_name="terminateNotificationProfile",
        )

        os_image_notification_profile = _schema_base_virtual_machine_profile_read.scheduled_events_profile.os_image_notification_profile
        os_image_notification_profile.enable = AAZBoolType()
        os_image_notification_profile.not_before_timeout = AAZStrType(
            serialized_name="notBeforeTimeout",
        )

        terminate_notification_profile = _schema_base_virtual_machine_profile_read.scheduled_events_profile.terminate_notification_profile
        terminate_notification_profile.enable = AAZBoolType()
        terminate_notification_profile.not_before_timeout = AAZStrType(
            serialized_name="notBeforeTimeout",
        )

        security_posture_reference = _schema_base_virtual_machine_profile_read.security_posture_reference
        security_posture_reference.exclude_extensions = AAZListType(
            serialized_name="excludeExtensions",
        )
        security_posture_reference.id = AAZStrType()
        security_posture_reference.is_overridable = AAZBoolType(
            serialized_name="isOverridable",
        )

        exclude_extensions = _schema_base_virtual_machine_profile_read.security_posture_reference.exclude_extensions
        exclude_extensions.Element = AAZStrType()

        security_profile = _schema_base_virtual_machine_profile_read.security_profile
        security_profile.encryption_at_host = AAZBoolType(
            serialized_name="encryptionAtHost",
        )
        security_profile.encryption_identity = AAZObjectType(
            serialized_name="encryptionIdentity",
        )
        security_profile.proxy_agent_settings = AAZObjectType(
            serialized_name="proxyAgentSettings",
        )
        security_profile.security_type = AAZStrType(
            serialized_name="securityType",
        )
        security_profile.uefi_settings = AAZObjectType(
            serialized_name="uefiSettings",
        )

        encryption_identity = _schema_base_virtual_machine_profile_read.security_profile.encryption_identity
        encryption_identity.user_assigned_identity_resource_id = AAZStrType(
            serialized_name="userAssignedIdentityResourceId",
        )

        proxy_agent_settings = _schema_base_virtual_machine_profile_read.security_profile.proxy_agent_settings
        proxy_agent_settings.enabled = AAZBoolType()
        proxy_agent_settings.key_incarnation_id = AAZIntType(
            serialized_name="keyIncarnationId",
        )
        proxy_agent_settings.mode = AAZStrType()

        uefi_settings = _schema_base_virtual_machine_profile_read.security_profile.uefi_settings
        uefi_settings.secure_boot_enabled = AAZBoolType(
            serialized_name="secureBootEnabled",
        )
        uefi_settings.v_tpm_enabled = AAZBoolType(
            serialized_name="vTpmEnabled",
        )

        service_artifact_reference = _schema_base_virtual_machine_profile_read.service_artifact_reference
        service_artifact_reference.id = AAZStrType()

        storage_profile = _schema_base_virtual_machine_profile_read.storage_profile
        storage_profile.data_disks = AAZListType(
            serialized_name="dataDisks",
        )
        storage_profile.disk_controller_type = AAZStrType(
            serialized_name="diskControllerType",
        )
        storage_profile.image_reference = AAZObjectType(
            serialized_name="imageReference",
        )
        storage_profile.os_disk = AAZObjectType(
            serialized_name="osDisk",
        )

        data_disks = _schema_base_virtual_machine_profile_read.storage_profile.data_disks
        data_disks.Element = AAZObjectType()

        _element = _schema_base_virtual_machine_profile_read.storage_profile.data_disks.Element
        _element.caching = AAZStrType()
        _element.create_option = AAZStrType(
            serialized_name="createOption",
            flags={"required": True},
        )
        _element.delete_option = AAZStrType(
            serialized_name="deleteOption",
        )
        _element.disk_iops_read_write = AAZIntType(
            serialized_name="diskIOPSReadWrite",
        )
        _element.disk_m_bps_read_write = AAZIntType(
            serialized_name="diskMBpsReadWrite",
        )
        _element.disk_size_gb = AAZIntType(
            serialized_name="diskSizeGB",
        )
        _element.lun = AAZIntType(
            flags={"required": True},
        )
        _element.managed_disk = AAZObjectType(
            serialized_name="managedDisk",
        )
        cls._build_schema_virtual_machine_scale_set_managed_disk_parameters_read(_element.managed_disk)
        _element.name = AAZStrType()
        _element.write_accelerator_enabled = AAZBoolType(
            serialized_name="writeAcceleratorEnabled",
        )

        image_reference = _schema_base_virtual_machine_profile_read.storage_profile.image_reference
        image_reference.community_gallery_image_id = AAZStrType(
            serialized_name="communityGalleryImageId",
        )
        image_reference.exact_version = AAZStrType(
            serialized_name="exactVersion",
            flags={"read_only": True},
        )
        image_reference.id = AAZStrType()
        image_reference.offer = AAZStrType()
        image_reference.publisher = AAZStrType()
        image_reference.shared_gallery_image_id = AAZStrType(
            serialized_name="sharedGalleryImageId",
        )
        image_reference.sku = AAZStrType()
        image_reference.version = AAZStrType()

        os_disk = _schema_base_virtual_machine_profile_read.storage_profile.os_disk
        os_disk.caching = AAZStrType()
        os_disk.create_option = AAZStrType(
            serialized_name="createOption",
            flags={"required": True},
        )
        os_disk.delete_option = AAZStrType(
            serialized_name="deleteOption",
        )
        os_disk.diff_disk_settings = AAZObjectType(
            serialized_name="diffDiskSettings",
        )
        os_disk.disk_size_gb = AAZIntType(
            serialized_name="diskSizeGB",
        )
        os_disk.image = AAZObjectType()
        os_disk.managed_disk = AAZObjectType(
            serialized_name="managedDisk",
        )
        cls._build_schema_virtual_machine_scale_set_managed_disk_parameters_read(os_disk.managed_disk)
        os_disk.name = AAZStrType()
        os_disk.os_type = AAZStrType(
            serialized_name="osType",
        )
        os_disk.vhd_containers = AAZListType(
            serialized_name="vhdContainers",
        )
        os_disk.write_accelerator_enabled = AAZBoolType(
            serialized_name="writeAcceleratorEnabled",
        )

        diff_disk_settings = _schema_base_virtual_machine_profile_read.storage_profile.os_disk.diff_disk_settings
        diff_disk_settings.option = AAZStrType()
        diff_disk_settings.placement = AAZStrType()

        image = _schema_base_virtual_machine_profile_read.storage_profile.os_disk.image
        image.uri = AAZStrType()

        vhd_containers = _schema_base_virtual_machine_profile_read.storage_profile.os_disk.vhd_containers
        vhd_containers.Element = AAZStrType()

        _schema.application_profile = cls._schema_base_virtual_machine_profile_read.application_profile
        _schema.capacity_reservation = cls._schema_base_virtual_machine_profile_read.capacity_reservation
        _schema.diagnostics_profile = cls._schema_base_virtual_machine_profile_read.diagnostics_profile
        _schema.extension_profile = cls._schema_base_virtual_machine_profile_read.extension_profile
        _schema.hardware_profile = cls._schema_base_virtual_machine_profile_read.hardware_profile
        _schema.license_type = cls._schema_base_virtual_machine_profile_read.license_type
        _schema.network_profile = cls._schema_base_virtual_machine_profile_read.network_profile
        _schema.os_profile = cls._schema_base_virtual_machine_profile_read.os_profile
        _schema.scheduled_events_profile = cls._schema_base_virtual_machine_profile_read.scheduled_events_profile
        _schema.security_posture_reference = cls._schema_base_virtual_machine_profile_read.security_posture_reference
        _schema.security_profile = cls._schema_base_virtual_machine_profile_read.security_profile
        _schema.service_artifact_reference = cls._schema_base_virtual_machine_profile_read.service_artifact_reference
        _schema.storage_profile = cls._schema_base_virtual_machine_profile_read.storage_profile
        _schema.time_created = cls._schema_base_virtual_machine_profile_read.time_created
        _schema.user_data = cls._schema_base_virtual_machine_profile_read.user_data

    _schema_disk_encryption_set_parameters_read = None

    @classmethod
    def _build_schema_disk_encryption_set_parameters_read(cls, _schema):
        if cls._schema_disk_encryption_set_parameters_read is not None:
            _schema.id = cls._schema_disk_encryption_set_parameters_read.id
            return

        cls._schema_disk_encryption_set_parameters_read = _schema_disk_encryption_set_parameters_read = AAZObjectType()

        disk_encryption_set_parameters_read = _schema_disk_encryption_set_parameters_read
        disk_encryption_set_parameters_read.id = AAZStrType()

        _schema.id = cls._schema_disk_encryption_set_parameters_read.id

    _schema_sub_resource_read = None

    @classmethod
    def _build_schema_sub_resource_read(cls, _schema):
        if cls._schema_sub_resource_read is not None:
            _schema.id = cls._schema_sub_resource_read.id
            return

        cls._schema_sub_resource_read = _schema_sub_resource_read = AAZObjectType()

        sub_resource_read = _schema_sub_resource_read
        sub_resource_read.id = AAZStrType()

        _schema.id = cls._schema_sub_resource_read.id

    _schema_vm_attribute_min_max_double_read = None

    @classmethod
    def _build_schema_vm_attribute_min_max_double_read(cls, _schema):
        if cls._schema_vm_attribute_min_max_double_read is not None:
            _schema.max = cls._schema_vm_attribute_min_max_double_read.max
            _schema.min = cls._schema_vm_attribute_min_max_double_read.min
            return

        cls._schema_vm_attribute_min_max_double_read = _schema_vm_attribute_min_max_double_read = AAZObjectType()

        vm_attribute_min_max_double_read = _schema_vm_attribute_min_max_double_read
        vm_attribute_min_max_double_read.max = AAZFloatType()
        vm_attribute_min_max_double_read.min = AAZFloatType()

        _schema.max = cls._schema_vm_attribute_min_max_double_read.max
        _schema.min = cls._schema_vm_attribute_min_max_double_read.min

    _schema_vm_attribute_min_max_integer_read = None

    @classmethod
    def _build_schema_vm_attribute_min_max_integer_read(cls, _schema):
        if cls._schema_vm_attribute_min_max_integer_read is not None:
            _schema.max = cls._schema_vm_attribute_min_max_integer_read.max
            _schema.min = cls._schema_vm_attribute_min_max_integer_read.min
            return

        cls._schema_vm_attribute_min_max_integer_read = _schema_vm_attribute_min_max_integer_read = AAZObjectType()

        vm_attribute_min_max_integer_read = _schema_vm_attribute_min_max_integer_read
        vm_attribute_min_max_integer_read.max = AAZIntType()
        vm_attribute_min_max_integer_read.min = AAZIntType()

        _schema.max = cls._schema_vm_attribute_min_max_integer_read.max
        _schema.min = cls._schema_vm_attribute_min_max_integer_read.min

    _schema_virtual_machine_scale_set_managed_disk_parameters_read = None

    @classmethod
    def _build_schema_virtual_machine_scale_set_managed_disk_parameters_read(cls, _schema):
        if cls._schema_virtual_machine_scale_set_managed_disk_parameters_read is not None:
            _schema.disk_encryption_set = cls._schema_virtual_machine_scale_set_managed_disk_parameters_read.disk_encryption_set
            _schema.security_profile = cls._schema_virtual_machine_scale_set_managed_disk_parameters_read.security_profile
            _schema.storage_account_type = cls._schema_virtual_machine_scale_set_managed_disk_parameters_read.storage_account_type
            return

        cls._schema_virtual_machine_scale_set_managed_disk_parameters_read = _schema_virtual_machine_scale_set_managed_disk_parameters_read = AAZObjectType()

        virtual_machine_scale_set_managed_disk_parameters_read = _schema_virtual_machine_scale_set_managed_disk_parameters_read
        virtual_machine_scale_set_managed_disk_parameters_read.disk_encryption_set = AAZObjectType(
            serialized_name="diskEncryptionSet",
        )
        cls._build_schema_disk_encryption_set_parameters_read(virtual_machine_scale_set_managed_disk_parameters_read.disk_encryption_set)
        virtual_machine_scale_set_managed_disk_parameters_read.security_profile = AAZObjectType(
            serialized_name="securityProfile",
        )
        virtual_machine_scale_set_managed_disk_parameters_read.storage_account_type = AAZStrType(
            serialized_name="storageAccountType",
        )

        security_profile = _schema_virtual_machine_scale_set_managed_disk_parameters_read.security_profile
        security_profile.disk_encryption_set = AAZObjectType(
            serialized_name="diskEncryptionSet",
        )
        cls._build_schema_disk_encryption_set_parameters_read(security_profile.disk_encryption_set)
        security_profile.security_encryption_type = AAZStrType(
            serialized_name="securityEncryptionType",
        )

        _schema.disk_encryption_set = cls._schema_virtual_machine_scale_set_managed_disk_parameters_read.disk_encryption_set
        _schema.security_profile = cls._schema_virtual_machine_scale_set_managed_disk_parameters_read.security_profile
        _schema.storage_account_type = cls._schema_virtual_machine_scale_set_managed_disk_parameters_read.storage_account_type


__all__ = ["List"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=unused-argument, line-too-long, import-outside-toplevel, raise-missing-from
from azure.cli.core.azclierror import InvalidArgumentValueError
from azure.core.paging import ItemPaged
from ._client_factory import cf_postgres_flexible_location_capabilities, cf_postgres_flexible_server_capabilities
from collections import defaultdict


def get_postgres_location_capability_info(cmd, location):
    list_location_capability_client = cf_postgres_flexible_location_capabilities(cmd.cli_ctx, '_')
    list_location_capability_result = list_location_capability_client.execute(location)
    return _postgres_parse_list_capability(list_location_capability_result)


def get_postgres_server_capability_info(cmd, resource_group, server_name):
    list_server_capability_client = cf_postgres_flexible_server_capabilities(cmd.cli_ctx, '_')
    list_server_capability_result = list_server_capability_client.list(resource_group_name=resource_group, server_name=server_name)
    return _postgres_parse_list_capability(list_server_capability_result)


def get_performance_tiers_for_storage(storage_edition, storage_size):
    performance_tiers = []
    storage_size_mb = None if storage_size is None else storage_size * 1024
    for storage_info in storage_edition.supported_storage_mb:
        if storage_size_mb == storage_info.storage_size_mb:
            for performance_tier in storage_info.supported_iops_tiers:
                performance_tiers.append(performance_tier.name)
    return performance_tiers


def get_performance_tiers(storage_edition):
    performance_tiers = []
    for storage_info in storage_edition.supported_storage_mb:
        for performance_tier in storage_info.supported_iops_tiers:
            if performance_tier.name not in performance_tiers:
                performance_tiers.append(performance_tier.name)
    return performance_tiers


def _postgres_parse_list_capability(result):
    result = _get_list_from_paged_response(result)

    if not result:
        raise InvalidArgumentValueError("No available SKUs in this location")

    supported_features = result[0].supported_features if result[0].supported_features is not None else []
    offer_restricted = [feature for feature in supported_features if feature.name == "OfferRestricted"]
    restricted = offer_restricted[0].status if offer_restricted else None
    zone_redundant = [feature for feature in supported_features if feature.name == "ZoneRedundantHa"]
    geo_backup = [feature for feature in supported_features if feature.name == "GeoBackup"]
    index_tuning = [feature for feature in supported_features if feature.name == "IndexTuning"]

    if restricted == "Enabled":
        raise InvalidArgumentValueError("The location is restricted for provisioning of flexible servers. Please try using another region.")

    if restricted != "Disabled":
        raise InvalidArgumentValueError("No available SKUs in this location.")

    single_az = zone_redundant[0].status != "Enabled" if zone_redundant else True
    geo_backup_supported = geo_backup[0].status == "Enabled" if geo_backup else False
    index_tuning_supported = index_tuning[0].status == "Enabled" if index_tuning else False

    tiers = result[0].supported_server_editions
    tiers_dict = {}
    for tier_info in tiers:
        tier_name = tier_info.name
        tier_dict = {}

        skus = set()
        zones = set()

        for sku in tier_info.supported_server_skus:
            skus.add(sku.name)
            for zone in sku.supported_zones:
                zones.add(zone)

        storage_sizes = set()
        for storage_edition in tier_info.supported_storage_editions:
            if storage_edition.name == "ManagedDisk":
                for storage_info in storage_edition.supported_storage_mb:
                    storage_sizes.add(int(storage_info.storage_size_mb // 1024))
                tier_dict["storage_edition"] = storage_edition
            elif storage_edition.name == "ManagedDiskV2" and len(storage_edition.supported_storage_mb) > 0:
                tier_dict["supported_storageV2_size"] = int(storage_edition.supported_storage_mb[0].storage_size_mb // 1024)
                tier_dict["supported_storageV2_size_max"] = int(storage_edition.supported_storage_mb[0].maximum_storage_size_mb // 1024)
                tier_dict["supported_storageV2_iops"] = storage_edition.supported_storage_mb[0].supported_iops
                tier_dict["supported_storageV2_iops_max"] = storage_edition.supported_storage_mb[0].supported_maximum_iops
                tier_dict["supported_storageV2_throughput"] = storage_edition.supported_storage_mb[0].supported_throughput
                tier_dict["supported_storageV2_throughput_max"] = storage_edition.supported_storage_mb[0].supported_maximum_throughput

        tier_dict["skus"] = skus
        tier_dict["storage_sizes"] = storage_sizes
        tiers_dict[tier_name] = tier_dict

    versions = set()
    for version in result[0].supported_server_versions:
        versions.add(version.name)

    supported_server_versions = defaultdict(list)
    for version in result[0].supported_server_versions:
        supported_server_versions[version.name] = version.supported_versions_to_upgrade

    return {
        'sku_info': tiers_dict,
        'single_az': single_az,
        'geo_backup_supported': geo_backup_supported,
        'zones': zones,
        'server_versions': versions,
        'supported_server_versions': supported_server_versions,
        'index_tuning_supported': index_tuning_supported
    }


def _get_list_from_paged_response(obj_list):
    return list(obj_list) if isinstance(obj_list, ItemPaged) else obj_list



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: disable=line-too-long
# pylint: disable=too-many-statements

from enum import Enum
from knack.arguments import CLIArgumentType
from azure.cli.core.commands.parameters import (get_enum_type,
                                                get_location_type,
                                                resource_group_name_type,
                                                get_three_state_flag)

from azure.mgmt.apimanagement.models import (SkuType, VirtualNetworkType, Protocol, ApiType, ProductState)
from azure.cli.command_modules.apim.actions import (TemplateParameter)


SKU_TYPES = SkuType
VNET_TYPES = VirtualNetworkType
API_PROTOCOLS = Protocol
API_TYPES = ApiType


class ImportFormat(Enum):
    Wadl = "Wadl"
    Swagger = "Swagger"
    OpenApi = "OpenApi"
    OpenApiJson = "OpenApiJson"
    Wsdl = "Wsdl"
    GraphQL = "GraphQL"


class ExportFormat(Enum):
    WadlFile = "WadlFile"
    SwaggerFile = "SwaggerFile"
    OpenApiFile = "OpenApiYamlFile"
    OpenApiJsonFile = "OpenApiJsonFile"
    WsdlFile = "WsdlFile"
    WadlUrl = "WadlUrl"
    SwaggerUrl = "SwaggerUrl"
    OpenApiUrl = "OpenApiYamlUrl"
    OpenApiJsonUrl = "OpenApiJsonUrl"
    WsdlUrl = "WsdlUrl"


def load_arguments(self, _):

    # REUSABLE ARGUMENT DEFINITIONS
    api_id = CLIArgumentType(arg_group='API',
                             help='API identifier. Must be unique in the current API Management service instance. Non-current revision has ;rev=n as a suffix where n is the revision number.')
    schema_id = CLIArgumentType(arg_group='Schema',
                                help='Schema identifier. Must be unique in the current API Management service instance. Non-current revision has ;rev=n as a suffix where n is the revision number.')

    from azure.cli.core.commands.parameters import tags_type
    from azure.cli.core.commands.validators import get_default_location_from_resource_group

    with self.argument_context('apim') as c:
        c.argument('resource_group_name', arg_type=resource_group_name_type)
        c.argument('tags', tags_type)
        c.argument('service_name', options_list=['--name', '-n'],
                   help="The name of the api management service instance", id_part=None)
        c.argument('name', options_list=['--name', '-n'],
                   help="The name of the api management service instance", id_part=None)
        c.argument('location', validator=get_default_location_from_resource_group)

    with self.argument_context('apim create') as c:
        c.argument('location', arg_type=get_location_type(self.cli_ctx),
                   validator=get_default_location_from_resource_group)
        c.argument(
            'publisher_name',
            help='The name of your organization for use in the developer portal and e-mail notifications.',
            required=True)
        c.argument('publisher_email', help='The e-mail address to receive all system notifications.')
        c.argument(
            'enable_client_certificate', arg_type=get_three_state_flag(),
            help='Enforces a client certificate to be presented on each request to the gateway and also enables the ability to authenticate the certificate in the policy on the gateway.')
        c.argument('virtual_network_type', get_enum_type(VNET_TYPES), options_list=[
                   '--virtual-network', '-v'], help='The virtual network type.')
        c.argument('sku_name', arg_type=get_enum_type(SKU_TYPES), help='The sku of the api management instance')
        c.argument('sku_capacity', type=int, help='The number of deployed units of the SKU.')
        c.argument('enable_managed_identity', arg_type=get_three_state_flag(),
                   help='Create a managed identity for the API Management service to access other Azure resources. Only meant to be used for Consumption SKU Service.')
        c.argument('public_network_access', arg_type=get_three_state_flag(),
                   help='Whether or not public endpoint access is allowed for this API Management service. If set to true, private endpoints are the exclusive access method.')
        c.argument('disable_gateway', arg_type=get_three_state_flag(),
                   help='Disable gateway in the master region. Only valid for an Api Management service deployed in multiple locations.')

    with self.argument_context('apim update') as c:
        c.argument(
            'publisher_name',
            help='The name of your organization for use in the developer portal and e-mail notifications.')
        c.argument('publisher_email', help='The e-mail address to receive all system notifications.')
        c.argument(
            'enable_client_certificate', arg_type=get_three_state_flag(),
            help='Enforces a client certificate to be presented on each request to the gateway and also enables the ability to authenticate the certificate in the policy on the gateway.')
        c.argument('virtual_network_type', get_enum_type(VNET_TYPES), options_list=[
                   '--virtual-network', '-v'], help='The virtual network type.')
        c.argument('sku_name', arg_type=get_enum_type(SKU_TYPES), help='The sku of the api management instance')
        c.argument('sku_capacity', type=int, help='The number of deployed units of the SKU.')
        c.argument('enable_managed_identity', arg_type=get_three_state_flag(),
                   help='Create a managed identity for the API Management service to access other Azure resources. Only meant to be used for Consumption SKU Service.')
        c.argument('public_network_access', arg_type=get_three_state_flag(),
                   help='Whether or not public endpoint access is allowed for this API Management service. If set to true, private endpoints are the exclusive access method.')
        c.argument('disable_gateway', arg_type=get_three_state_flag(),
                   help='Disable gateway in the master region. Only valid for an Api Management service deployed in multiple locations.')

    with self.argument_context('apim backup') as c:
        c.argument('backup_name', help='The name of the backup file to create.')
        c.argument('storage_account_name', arg_group='Storage',
                   help='The name of the storage account used to place the backup.')
        c.argument('storage_account_key', arg_group='Storage',
                   help='The access key of the storage account used to place the backup.')
        c.argument(
            'storage_account_container', options_list=['--storage-account-container', '--container-name'],
            help='The name of the storage account container used to place the backup.')

    with self.argument_context('apim restore') as c:
        c.argument('backup_name', help='The name of the backup file to restore.')
        c.argument('storage_account_name', arg_group='Storage',
                   help='The name of the storage account used to retrieve the backup from.')
        c.argument('storage_account_key', arg_group='Storage',
                   help='The access key of the storage account used to retrieve the backup from.')
        c.argument(
            'storage_account_container', options_list=['--storage-account-container', '--container-name'],
            help='The name of the storage account container used to retrieve the backup from.')

    with self.argument_context('apim api show') as c:
        c.argument('service_name', options_list=['--service-name'],
                   help='The name of the API Management service instance.')
        c.argument(
            'api_id', arg_group='API',
            help='API identifier. Must be unique in the current API Management service instance. Non-current revision has ;rev=n as a suffix where n is the revision number.')

    with self.argument_context('apim api list') as c:
        c.argument('service_name', options_list=['--service-name', '-n'],
                   help='The name of the API Management service instance.')
        c.argument('filter_display_name', arg_group='API', help='Filter of APIs by displayName.')
        c.argument('skip', type=int, help='Number of records to skip.')
        c.argument('top', type=int, help='Number of records to return.')

    with self.argument_context('apim api schema create') as c:
        c.argument('service_name', options_list=['--service-name', '-n'],
                   help='The name of the API Management service instance.')
        c.argument('schema_name',
                   help='The name of the schema resource.')
        c.argument(
            'api_id', arg_type=api_id, required=True)
        c.argument('schema_path', help='File path specified to import schema of the API.')
        c.argument('schema_content', help='Json escaped string defining the document representing the Schema')
        c.argument(
            'schema_id', arg_type=schema_id,
            required=True)
        c.argument(
            'schema_type', arg_group='Schema',
            help='Schema content type. Must be a valid media type used in a Content-Type header as defined in the RFC 2616. Media type of the schema document (e.g. application/json, application/xml).',
            required=True)
        c.argument(
            'resource_type', arg_group='Schema',
            help='The type of the resource. E.g. "Microsoft.Compute/virtualMachines" or "Microsoft.Storage/storageAccounts".')
        c.argument('if_match', help='ETag of the Entity.')

    with self.argument_context('apim api schema delete') as c:
        c.argument('service_name', options_list=['--service-name', '-n'],
                   help='The name of the API Management service instance.')
        c.argument(
            'api_id', arg_type=api_id,
            required=True)
        c.argument(
            'schema_id', arg_type=schema_id,
            required=True)
        c.argument('if_match', help='ETag of the Entity.')

    with self.argument_context('apim api schema show') as c:
        c.argument('service_name', options_list=['--service-name', '-n'],
                   help='The name of the API Management service instance.')
        c.argument(
            'api_id', arg_type=api_id,
            required=True)
        c.argument(
            'schema_id', arg_type=schema_id,
            required=True)

    with self.argument_context('apim api schema list') as c:
        c.argument('service_name', options_list=['--service-name', '-n'],
                   help='The name of the API Management service instance.')
        c.argument('filter_display_name', arg_group='Schema', help='Filter of APIs by displayName.')
        c.argument('skip', type=int, help='Number of records to skip.')
        c.argument('top', type=int, help='Number of records to return.')
        c.argument(
            'api_id', arg_type=api_id,
            required=True)

    with self.argument_context('apim api schema get-etag') as c:
        c.argument('service_name', options_list=['--service-name', '-n'],
                   help='The name of the API Management service instance.')
        c.argument(
            'api_id', arg_type=api_id,
            required=True)
        c.argument(
            'schema_id', arg_type=schema_id,
            required=True)

    with self.argument_context('apim api create') as c:
        c.argument('service_name', options_list=['--service-name', '-n'],
                   help='The name of the API Management service instance.')
        c.argument(
            'api_id', arg_type=api_id,
            required=True)
        c.argument(
            'path', arg_group='API',
            help='Required. Relative URL uniquely identifying this API and all of its resource paths within the API Management service instance.',
            required=True)
        c.argument('display_name', arg_group='API', help='API name. Must be 1 to 300 characters long.', required=True)
        c.argument('description', arg_group='API', help='Description of the API. May include HTML formatting tags.')
        c.argument('subscription_key_header_name', arg_group='API', help='Specifies the subscription key header name.')
        c.argument('subscription_key_query_param_name', arg_group='API',
                   help='Specifies the subscription key query string parameter name.')
        c.argument('open_id_provider_id', help='Specifies the openid in the authentication setting.')
        c.argument('bearer_token_sending_methods', nargs='+', help='Specifies the sending methods for bearer token.')
        c.argument('authorization_server_id', help='Specifies the OAuth authorization server ID.')
        c.argument('authorization_scope', help='Specifies the OAuth operations scope.')
        c.argument(
            'service_url', arg_group='API',
            help='Absolute URL of the backend service implementing this API. Cannot be more than 2000 characters long.')
        c.argument('protocols', arg_group='API', arg_type=get_enum_type(API_PROTOCOLS), nargs='+',
                   help='Describes on which protocols the operations in this API can be invoked.')
        c.argument('api_type', arg_group='API', arg_type=get_enum_type(API_TYPES), help='The type of the API.')
        c.argument('subscription_required', arg_group='API', arg_type=get_three_state_flag(),
                   help='If true, the API requires a subscription key on requests.')
        c.argument(
            'subscription_key_required',
            help='Specifies whether subscription key is required during call to this API, true - API is included into closed products only, false - API is included into open products alone, null - there is a mix of products.')
        c.argument('tags', tags_type)

    with self.argument_context('apim api delete') as c:
        c.argument('service_name', options_list=['--service-name', '-n'],
                   help='The name of the API Management service instance.')
        c.argument(
            'api_id', arg_type=api_id)
        c.argument('delete_revisions', help='Delete all revisions of the Api.')
        c.argument('if_match', help='ETag of the Entity.')

    with self.argument_context('apim api update') as c:
        c.argument('service_name', options_list=['--service-name', '-n'],
                   help='The name of the API Management service instance.')
        c.argument(
            'api_id', arg_type=api_id,
            required=True)
        c.argument('display_name', arg_group='API', help='API name. Must be 1 to 300 characters long.')
        c.argument(
            'path', arg_group='API',
            help='Required. Relative URL uniquely identifying this API and all of its resource paths within the API Management service instance.')
        c.argument('description', arg_group='API', help='Description of the API. May include HTML formatting tags.')
        c.argument('subscription_key_header_name', arg_group='API', help='Specifies the subscription key header name.')
        c.argument('subscription_key_query_param_name', arg_group='API',
                   help='Specifies the subscription key query string parameter name.')
        c.argument(
            'service_url', arg_group='API',
            help='Absolute URL of the backend service implementing this API. Cannot be more than 2000 characters long.')
        c.argument('protocols', arg_group='API', arg_type=get_enum_type(API_PROTOCOLS), nargs='+',
                   help='Describes on which protocols the operations in this API can be invoked.')
        c.argument('api_type', arg_group='API', arg_type=get_enum_type(API_TYPES), help='The type of the API.')
        c.argument('subscription_required', arg_group='API', arg_type=get_three_state_flag(),
                   help='If true, the API requires a subscription key on requests.')
        c.argument('tags', tags_type)

    with self.argument_context('apim api import') as c:
        c.argument('service_name', options_list=['--service-name', '-n'],
                   help="The name of the api management service instance", id_part=None)
        c.argument(
            'path', arg_group='API',
            help='Required. Relative URL uniquely identifying this API and all of its resource paths within the API Management service instance.',
            required=True)
        c.argument('specification_format', arg_type=get_enum_type(
            ImportFormat), help='Specify the format of the imported API.')
        c.argument('description', arg_group='API', help='Description of the API. May include HTML formatting tags.')
        c.argument('subscription_key_header_name', arg_group='API', help='Specifies the subscription key header name.')
        c.argument('subscription_key_query_param_name', arg_group='API',
                   help='Specifies the subscription key query string parameter name.')
        c.argument(
            'api_id', arg_type=api_id)
        c.argument(
            'api_revision', arg_group='API',
            help='Describes the Revision of the Api. If no value is provided, default revision 1 is created.')
        c.argument(
            'api_version', arg_group='API',
            help='Describes the Version of the Api. If you add a version to a non-versioned API, an Original version will be automatically created and will respond on the default URL')
        c.argument('api_version_set_id', arg_group='API', help='Describes the Version Set to be used with the API')
        c.argument(
            'service_url', arg_group='API',
            help='Absolute URL of the backend service implementing this API. Cannot be more than 2000 characters long.')
        c.argument('protocols', arg_group='API', arg_type=get_enum_type(API_PROTOCOLS), nargs='+',
                   help='Describes on which protocols(one or more) the operations in this API can be invoked.')
        c.argument('api_type', arg_group='API', arg_type=get_enum_type(API_TYPES), help='The type of the API.')
        c.argument('subscription_required', arg_group='API', arg_type=get_three_state_flag(),
                   help='If true, the API requires a subscription key on requests.')
        c.argument('display_name', arg_group='API', help='Display name of this API.')
        c.argument('specification_path', help='File path specified to import the API.')
        c.argument('specification_url', help='Url specified to import the API.')
        c.argument('soap_api_type', help='The type of API when file format is WSDL.')
        c.argument('wsdl_service_name', help='Local name of WSDL Service to be imported.')
        c.argument('wsdl_endpoint_name', help='Local name of WSDL Endpoint (port) to be imported.')

    with self.argument_context('apim api export') as c:
        c.argument('resource_group_name', arg_type=resource_group_name_type,
                   help="The name of the resource group. The name is case insensitive.")
        c.argument('service_name', options_list=['--service-name', '-n'],
                   help="The name of the api management service instance", id_part=None)
        c.argument('api_id', arg_type=api_id,
                   help='API identifier. Must be unique in the current API Management service instance. Non-current revision has ;rev=n as a suffix where n is the revision number. Regex pattern: ^[^*#&+:<>?]+$')
        c.argument('export_format', arg_type=get_enum_type(ExportFormat), options_list=['--export-format', '--ef'],
                   help='Specify the format of the exporting API.')
        c.argument('file_path', options_list=['--file-path', '-f'],
                   help='File path specified to export the API.')

    with self.argument_context('apim product api list') as c:
        c.argument('service_name', options_list=['--service-name', '-n'],
                   help="The name of the api management service instance", id_part=None)
        c.argument('product_id', help="Product identifier. Must be unique in the current API Management service instance.")

    with self.argument_context('apim product api check') as c:
        c.argument('service_name', options_list=['--service-name', '-n'],
                   help="The name of the api management service instance", id_part=None)
        c.argument('product_id', help="Product identifier. Must be unique in the current API Management service instance.")
        c.argument(
            'api_id', arg_type=api_id)

    with self.argument_context('apim product api add') as c:
        c.argument('service_name', options_list=['--service-name', '-n'],
                   help="The name of the api management service instance", id_part=None)
        c.argument('product_id', help="Product identifier. Must be unique in the current API Management service instance.")
        c.argument(
            'api_id', arg_type=api_id)

    with self.argument_context('apim product api delete') as c:
        c.argument('service_name', options_list=['--service-name', '-n'],
                   help="The name of the api management service instance", id_part=None)
        c.argument('product_id', help="Product identifier. Must be unique in the current API Management service instance.")
        c.argument(
            'api_id', arg_type=api_id)

    with self.argument_context('apim product') as c:
        c.argument('service_name', options_list=['--service-name', '-n'],
                   help="The name of the api management service instance", id_part=None)
        c.argument('product_id', help="Product identifier. Must be unique in the current API Management service instance.")
        c.argument('product_name', help="Product name.")
        c.argument('description', help="Product description. May include HTML formatting tags.")
        c.argument(
            'legal_terms',
            help="Product terms of use. Developers trying to subscribe to the product will be presented and required to accept these terms before they can complete the subscription process.")
        c.argument(
            'subscription_required', options_list=['--subscription-required', '-s'], arg_type=get_three_state_flag(),
            help="Whether a product subscription is required for accessing APIs included in this product.")
        c.argument('approval_required', arg_type=get_three_state_flag(), help="whether subscription approval is required. If false, new subscriptions will be approved automatically enabling developers to call the product’s APIs immediately after subscribing. If true, administrators must manually approve the subscription before the developer can use any of the product’s APIs. Can be present only if subscriptionRequired property is present and has a value of false.")
        c.argument('subscriptions_limit', help="Whether the number of subscriptions a user can have to this product at the same time. Set to null or omit to allow unlimited per user subscriptions. Can be present only if subscriptionRequired property is present and has a value of false.")
        c.argument(
            'state', arg_type=get_enum_type(ProductState),
            help="whether product is published or not. Published products are discoverable by users of developer portal. Non published products are visible only to administrators. Default state of Product is notPublished. Possible values include: 'notPublished', 'published'")
        c.argument('delete_subscriptions', help="Delete existing subscriptions associated with the product or not.")
        c.argument('if_match', help='ETag of the Entity.')

    with self.argument_context('apim nv') as c:
        c.argument('service_name', options_list=['--service-name', '-n'],
                   help='The name of the API Management service instance.')
        c.argument('named_value_id', help='Identifier of the NamedValue.')
        c.argument(
            'display_name',
            help='Required. Unique name of NamedValue. It may contain only letters, digits, period, dash, and underscore characters.')
        c.argument('value', help='Required. Value of the NamedValue. Can contain policy expressions. It may not be empty or consist only of whitespace. This property will not be filled on GET operations! Use /listSecrets POST request to get the value.')
        c.argument(
            'secret', arg_type=get_three_state_flag(),
            help='Determines whether the value is a secret and should be encrypted or not. Default value is false.')
        c.argument('tags', tags_type)
        c.argument('if_match', help='ETag of the Entity.')

    with self.argument_context('apim api operation') as c:
        c.argument('service_name', options_list=['--service-name', '-n'],
                   help="The name of the API Management service instance.")
        c.argument(
            'api_id', arg_type=api_id)
        c.argument(
            'operation_id',
            help='Operation identifier within an API. Must be unique in the current API Management service instance.')
        c.argument('description', help="Description of the operation. May include HTML formatting tags.")
        c.argument(
            'template_parameters', options_list=['--template-parameters', '--params', '-p'],
            action=TemplateParameter, nargs='+', help="Collection of URL template parameters.")
        c.argument(
            'method',
            help="Required. A Valid HTTP Operation Method. Typical Http Methods like GET, PUT, POST but not limited by only them.")
        c.argument('display_name', help="Required. Operation Name.")
        c.argument('if_match', help='ETag of the Entity.')
        c.argument(
            'url_template',
            help='Relative URL template identifying the target resource for this operation. May include parameters.')

    with self.argument_context('apim api release') as c:
        c.argument('service_name', options_list=['--service-name', '-n'],
                   help="The name of the API Management service instance.")
        c.argument(
            'api_id',
            arg_type=api_id)
        c.argument(
            'release_id',
            help="Release identifier within an API. Must be unique in the current API Management service instance.")
        c.argument('api_revision', help="API revision number.")
        c.argument('notes', help="Release Notes.")
        c.argument('if_match', help='ETag of the Entity.')

    with self.argument_context('apim api revision') as c:
        c.argument('service_name', options_list=['--service-name', '-n'],
                   help="The name of the API Management service instance.")
        c.argument(
            'api_id',
            arg_type=api_id)
        c.argument('api_revision', help='Describes the Revision of the Api.')
        c.argument('api_revision_description', options_list=[
                   '--api-revision-description', '--rev-description'], help='Description of the Api Revision.')

    with self.argument_context('apim api versionset') as c:
        c.argument('version_set_id', help="A resource identifier for the related ApiVersionSet.")
        c.argument(
            'versioning_scheme',
            help="Required. An value that determines where the API Version identifer will be located in a HTTP request. Possible values include: 'Segment', 'Query', 'Header'")
        c.argument('display_name', help="Required. Name of API Version Set")
        c.argument('service_name', options_list=['--service-name', '-n'],
                   help="The name of the API Management service instance.")
        c.argument('description', help="Description of API Version Set.")
        c.argument(
            'version_query_name',
            help="Name of query parameter that indicates the API Version if versioningScheme is set to `query`.")
        c.argument(
            'version_header_name',
            help="Name of HTTP header parameter that indicates the API Version if versioningScheme is set to `header`.")
        c.argument('if_match', help='ETag of the Entity.')

    with self.argument_context('apim deletedservice show') as c:
        c.argument('location', arg_type=get_location_type(self.cli_ctx))
        c.argument('service_name', options_list=['--service-name', '-n'],
                   help="The name of the soft deleted API Management service instance.")

    with self.argument_context('apim deletedservice purge') as c:
        c.argument('location', arg_type=get_location_type(self.cli_ctx))
        c.argument(
            'service_name', options_list=['--service-name', '-n'],
            help="The name of the soft deleted API Management service instance.")

    with self.argument_context('apim graphql resolver create') as c:
        c.argument('service_name', options_list=['--service-name', '-n'],
                   help='The name of the API Management service instance.')
        c.argument('api_id', arg_type=api_id)
        c.argument('resolver_id',
                   help='Resolver identifier within a GraphQL API. Must be unique in the current API Management service instance.')
        c.argument('display_name',
                   help='Resolver Name.')
        c.argument('path', help='Resolver identifier within a GraphQL API. Must be unique in the current API Management service instance.', required=True)
        c.argument('description',
                   help='Description of the resolver. May include HTML formatting tags.')

    with self.argument_context('apim graphql resolver delete') as c:
        c.argument('service_name', options_list=['--service-name', '-n'],
                   help='The name of the API Management service instance.')
        c.argument('api_id', arg_type=api_id)
        c.argument('resolver_id', help='Resolver identifier within a GraphQL API. Must be unique in the current API Management service instance.')
        c.argument('if_match', help='ETag of the Entity.')

    with self.argument_context('apim graphql resolver show') as c:
        c.argument('service_name', options_list=['--service-name', '-n'],
                   help='The name of the API Management service instance.')
        c.argument('api_id', arg_type=api_id)
        c.argument('resolver_id', help='Resolver identifier within a GraphQL API. Must be unique in the current API Management service instance.')

    with self.argument_context('apim graphql resolver list') as c:
        c.argument('service_name', options_list=['--service-name', '-n'],
                   help='The name of the API Management service instance.')
        c.argument('api_id', arg_type=api_id)

    with self.argument_context('apim graphql resolver policy create') as c:
        c.argument('service_name', options_list=['--service-name', '-n'],
                   help='The name of the API Management service instance.')
        c.argument('api_id', arg_type=api_id)
        c.argument('resolver_id', help='Resolver identifier within a GraphQL API. Must be unique in the current API Management service instance.')
        c.argument('policy_format', help='Format of the policyContent.')
        c.argument('value_path', help='Contents of the Policy as defined by the format.')

    with self.argument_context('apim graphql resolver policy show') as c:
        c.argument('service_name', options_list=['--service-name', '-n'],
                   help='The name of the API Management service instance.')
        c.argument('api_id', arg_type=api_id)
        c.argument('resolver_id', help='Resolver identifier within a GraphQL API. Must be unique in the current API Management service instance.')

    with self.argument_context('apim graphql resolver policy list') as c:
        c.argument('service_name', options_list=['--service-name', '-n'],
                   help='The name of the API Management service instance.')
        c.argument('api_id', arg_type=api_id)
        c.argument('resolver_id', help='Resolver identifier within a GraphQL API. Must be unique in the current API Management service instance.')

    with self.argument_context('apim graphql resolver policy delete') as c:
        c.argument('service_name', options_list=['--service-name', '-n'],
                   help='The name of the API Management service instance.')
        c.argument('api_id', arg_type=api_id)
        c.argument('resolver_id', help='Resolver identifier within a GraphQL API. Must be unique in the current API Management service instance.')
        c.argument('if_match', help='ETag of the Entity.')
